@@ -1,9 +1,11 @@
 /** @file check_matrices.cpp
  *
- *  Here we test manipulations on GiNaC's symbolic matrices. */
+ *  Here we test manipulations on GiNaC's symbolic matrices.  They are a
+ *  well-tried resource for cross-checking the underlying symbolic
+ *  manipulations. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,12 +26,12 @@
 
 /* determinants of some sparse symbolic matrices with coefficients in
  * an integral domain. */
-static unsigned integdom_matrix_determinants(void)
+static unsigned integdom_matrix_determinants()
 {
 	unsigned result = 0;
 	symbol a("a");
 	
-	for (unsigned size=3; size<20; ++size) {
+	for (unsigned size=3; size<22; ++size) {
 		matrix A(size,size);
 		// populate one element in each row:
 		for (unsigned r=0; r<size-1; ++r)
@@ -51,12 +53,12 @@ static unsigned integdom_matrix_determinants(void)
 
 /* determinants of some symbolic matrices with multivariate rational function
  * coefficients. */
-static unsigned rational_matrix_determinants(void)
+static unsigned rational_matrix_determinants()
 {
 	unsigned result = 0;
 	symbol a("a"), b("b"), c("c");
 	
-	for (unsigned size=3; size<8; ++size) {
+	for (unsigned size=3; size<9; ++size) {
 		matrix A(size,size);
 		for (unsigned r=0; r<size-1; ++r) {
 			// populate one or two elements in each row:
@@ -85,12 +87,12 @@ static unsigned rational_matrix_determinants(void)
 }
 
 /* Some quite funny determinants with functions and stuff like that inside. */
-static unsigned funny_matrix_determinants(void)
+static unsigned funny_matrix_determinants()
 {
 	unsigned result = 0;
 	symbol a("a"), b("b"), c("c");
 	
-	for (unsigned size=3; size<7; ++size) {
+	for (unsigned size=3; size<8; ++size) {
 		matrix A(size,size);
 		for (unsigned co=0; co<size-1; ++co) {
 			// populate one or two elements in each row:
@@ -119,12 +121,12 @@ static unsigned funny_matrix_determinants(void)
 }
 
 /* compare results from different determinant algorithms.*/
-static unsigned compare_matrix_determinants(void)
+static unsigned compare_matrix_determinants()
 {
 	unsigned result = 0;
 	symbol a("a");
 	
-	for (unsigned size=2; size<7; ++size) {
+	for (unsigned size=2; size<8; ++size) {
 		matrix A(size,size);
 		for (unsigned co=0; co<size; ++co) {
 			for (unsigned ro=0; ro<size; ++ro) {
@@ -156,12 +158,12 @@ static unsigned compare_matrix_determinants(void)
 	return result;
 }
 
-static unsigned symbolic_matrix_inverse(void)
+static unsigned symbolic_matrix_inverse()
 {
 	unsigned result = 0;
 	symbol a("a"), b("b"), c("c");
 	
-	for (unsigned size=2; size<5; ++size) {
+	for (unsigned size=2; size<6; ++size) {
 		matrix A(size,size);
 		do {
 			for (unsigned co=0; co<size; ++co) {
@@ -193,7 +195,7 @@ static unsigned symbolic_matrix_inverse(void)
 	return result;
 }
 
-unsigned check_matrices(void)
+unsigned check_matrices()
 {
 	unsigned result = 0;
 	
