@@ -4,7 +4,7 @@
  *  tests on these numbers like is_integer() etc... */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 /* Simple and maybe somewhat pointless consistency tests of assorted tests and
  * conversions. */
-static unsigned check_numeric1(void)
+static unsigned check_numeric1()
 {
 	unsigned result = 0;
 	bool errorflag = false;
@@ -53,7 +53,7 @@ static unsigned check_numeric1(void)
 	return result;
 }
 
-static unsigned check_numeric2(void)
+static unsigned check_numeric2()
 {
 	unsigned result = 0;
 	bool errorflag = false;
@@ -105,7 +105,7 @@ static unsigned check_numeric2(void)
 	return result;
 }
 
-unsigned check_numeric(void)
+unsigned check_numeric()
 {
 	unsigned result = 0;
 	
