@@ -3,7 +3,7 @@
  *  Here we test GiNaC's archiving system. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 
 #include <fstream>
 
-unsigned exam_archive(void)
+unsigned exam_archive()
 {
 	unsigned result = 0;
 	
@@ -40,15 +40,15 @@ unsigned exam_archive(void)
 	    * epsilon_tensor(idx(fail(), 3), idx(0, 3), idx(y/2, 3))
 	  + lorentz_g(
 	      varidx(lst(x, -11*y, acos(2*x).series(x==3-5*I, 3)) * color_ONE()
-	        * metric_tensor(spinidx(0, 5, false, true), varidx(2, 4)), zeta(3)),
+	        * metric_tensor(varidx(log(cos(128.0/(x*y))), 5), varidx(2, 5)), zeta(3)),
 	      varidx(diag_matrix(lst(-1, Euler, atan(x/y==-15*I/17)))
-	        * delta_tensor(idx(x, 2), idx(wild(7), 3)), log(cos(128.0/(x*y))), true),
+	        * delta_tensor(idx(x, 2), idx(wild(7), 3)), zeta(3), true),
 	      true
 	    )
 	  + dirac_gamma(varidx(mu, dim)) * dirac_gamma(varidx(mu, 4-dim, true))
 	    * color_T(idx(x, 8), 1) * color_h(idx(x, 8), idx(y, 8), idx(2, 8))
 	    * indexed(x, sy_anti(), idx(2*y+1, x), varidx(-mu, 5))
-	  - 2.4275 * spinor_metric(spinidx(x), spinidx(y))
+	  - 2.4275 * spinor_metric(spinidx(0, 2, false, true), spinidx(y))
 	  + abs(x).series(x == y, 4);
 
 	archive ar;
