@@ -4,7 +4,7 @@
  *  functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 #include "exams.h"
 
 /* Assorted tests on other transcendental functions. */
-static unsigned inifcns_consist_trans(void)
+static unsigned inifcns_consist_trans()
 {
+	using GiNaC::asin; using GiNaC::acos;
+
 	unsigned result = 0;
 	symbol x("x");
 	ex chk;
@@ -116,7 +118,7 @@ static unsigned inifcns_consist_trans(void)
 
 /* Simple tests on the tgamma function.  We stuff in arguments where the results
  * exists in closed form and check if it's ok. */
-static unsigned inifcns_consist_gamma(void)
+static unsigned inifcns_consist_gamma()
 {
 	unsigned result = 0;
 	ex e;
@@ -161,8 +163,10 @@ static unsigned inifcns_consist_gamma(void)
 
 /* Simple tests on the Psi-function (aka polygamma-function).  We stuff in
    arguments where the result exists in closed form and check if it's ok. */
-static unsigned inifcns_consist_psi(void)
+static unsigned inifcns_consist_psi()
 {
+	using GiNaC::log;
+
 	unsigned result = 0;
 	symbol x;
 	ex e, f;
@@ -183,7 +187,7 @@ static unsigned inifcns_consist_psi(void)
 /* Simple tests on the Riemann Zeta function.  We stuff in arguments where the
  * result exists in closed form and check if it's ok.  Of course, this checks
  * the Bernoulli numbers as a side effect. */
-static unsigned inifcns_consist_zeta(void)
+static unsigned inifcns_consist_zeta()
 {
 	unsigned result = 0;
 	ex e;
@@ -208,7 +212,7 @@ static unsigned inifcns_consist_zeta(void)
 	return result;
 }
 
-unsigned exam_inifcns(void)
+unsigned exam_inifcns()
 {
 	unsigned result = 0;
 	
