@@ -3,7 +3,7 @@
  *  Prototypes for all individual exams. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@ unsigned exam_paranoia();
 unsigned exam_numeric();
 unsigned exam_powerlaws();
 unsigned exam_inifcns();
+unsigned exam_inifcns_nstdsums();
 unsigned exam_differentiation();
 unsigned exam_polygcd();
 unsigned exam_normalization();
@@ -43,6 +44,8 @@ unsigned exam_indexed();
 unsigned exam_color();
 unsigned exam_clifford();
 unsigned exam_archive();
+unsigned exam_structure();
+unsigned exam_hashmap();
 unsigned exam_misc();
 
 #endif // ndef EXAMS_H
