@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 
 #include "times.h"
 
-static unsigned test(void)
+static unsigned test()
 {
 	numeric s;
 	
@@ -37,7 +37,7 @@ static unsigned test(void)
 	return 0;
 }
 
-unsigned time_lw_B(void)
+unsigned time_lw_B()
 {
 	unsigned result = 0;
 	unsigned count = 0;
