@@ -4,7 +4,7 @@
  *  by Robert H. Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ static unsigned test(unsigned n)
 	return result;
 }
 
-unsigned time_lw_IJKL(void)
+unsigned time_lw_IJKL()
 {
 	unsigned result = 0;
 	
