@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
 
 #include "times.h"
 
-static const bool do_test = false;  // set to true in order to run this beast
+static const bool do_test = true;  // set to true in order to run this beast
 
-static unsigned test(void)
+static unsigned test()
 {
 	// same matrix as in test P':
 	const unsigned n = 10;
@@ -66,7 +66,7 @@ static unsigned test(void)
 	return 0;
 }
 
-unsigned time_lw_Qprime(void)
+unsigned time_lw_Qprime()
 {
 	unsigned result = 0;
 	unsigned count = 0;
