@@ -7,7 +7,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ static unsigned vandermonde_det(unsigned size)
 	return result;
 }
 
-unsigned time_vandermonde(void)
+unsigned time_vandermonde()
 {
 	unsigned result = 0;
 	
