@@ -57,7 +57,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(basic, void,
  *  tinfo_key and the hash value. */
 basic::basic(const basic & other) : tinfo_key(other.tinfo_key), flags(other.flags & ~status_flags::dynallocated), hashvalue(other.hashvalue)
 {
-	GINAC_ASSERT(typeid(*this) == typeid(other));
 }
 
 /** basic assignment operator: the other object might be of a derived class. */
@@ -208,7 +207,7 @@ void basic::do_print_python_repr(const print_python_repr & c, unsigned level) co
  *  @see basic::dbgprinttree */
 void basic::dbgprint() const
 {
-	this->print(std::cerr);
+	this->print(print_dflt(std::cerr));
 	std::cerr << std::endl;
 }
 
@@ -307,12 +306,23 @@ ex basic::map(map_function & f) const
 	if (num == 0)
 		return *this;
 
-	basic *copy = duplicate();
-	copy->setflag(status_flags::dynallocated);
-	copy->clearflag(status_flags::hash_calculated | status_flags::expanded);
-	for (size_t i=0; i<num; i++)
-		copy->let_op(i) = f(copy->op(i));
-	return *copy;
+	basic *copy = NULL;
+	for (size_t i=0; i<num; i++) {
+		const ex & o = op(i);
+		const ex & n = f(o);
+		if (!are_ex_trivially_equal(o, n)) {
+			if (copy == NULL)
+				copy = duplicate();
+			copy->let_op(i) = n;
+		}
+	}
+
+	if (copy) {
+		copy->setflag(status_flags::dynallocated);
+		copy->clearflag(status_flags::hash_calculated | status_flags::expanded);
+		return *copy;
+	} else
+		return *this;
 }
 
 /** Return degree of highest power in object s. */
@@ -472,6 +482,20 @@ ex basic::evalm() const
 		return map(map_evalm);
 }
 
+/** Function object to be applied by basic::eval_integ(). */
+struct eval_integ_map_function : public map_function {
+	ex operator()(const ex & e) { return eval_integ(e); }
+} map_eval_integ;
+
+/** Evaluate integrals, if result is known. */
+ex basic::eval_integ() const
+{
+	if (nops() == 0)
+		return *this;
+	else
+		return map(map_eval_integ);
+}
+
 /** Perform automatic symbolic evaluations on indexed expression that
  *  contains this object as the base expression. */
 ex basic::eval_indexed(const basic & i) const
@@ -486,7 +510,7 @@ ex basic::eval_indexed(const basic & i) const
  *  (or a subclass) and their indices are compatible. This function is used
  *  internally by simplify_indexed().
  *
- *  @param self First indexed expression; it's base object is *this
+ *  @param self First indexed expression; its base object is *this
  *  @param other Second indexed expression
  *  @return sum of self and other 
  *  @see ex::simplify_indexed() */
@@ -498,7 +522,7 @@ ex basic::add_indexed(const ex & self, const ex & other) const
 /** Multiply an indexed expression with a scalar. This function is used
  *  internally by simplify_indexed().
  *
- *  @param self Indexed expression; it's base object is *this
+ *  @param self Indexed expression; its base object is *this
  *  @param other Numeric value
  *  @return product of self and other
  *  @see ex::simplify_indexed() */
@@ -514,7 +538,7 @@ ex basic::scalar_mul_indexed(const ex & self, const numeric & other) const
  *  and that at least one dummy index has been found. This functions is
  *  used internally by simplify_indexed().
  *
- *  @param self Pointer to first indexed expression; it's base object is *this
+ *  @param self Pointer to first indexed expression; its base object is *this
  *  @param other Pointer to second indexed expression
  *  @param v The complete vector of factors
  *  @return true if the contraction was successful, false otherwise
@@ -549,8 +573,8 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 	if (is_exactly_a<wildcard>(pattern)) {
 
 		// Wildcard matches anything, but check whether we already have found
-		// a match for that wildcard first (if so, it the earlier match must
-		// be the same expression)
+		// a match for that wildcard first (if so, the earlier match must be
+		// the same expression)
 		for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it) {
 			if (it->op(0).is_equal(pattern))
 				return is_equal(ex_to<basic>(it->op(1)));
@@ -781,7 +805,7 @@ unsigned basic::calchash() const
 struct expand_map_function : public map_function {
 	unsigned options;
 	expand_map_function(unsigned o) : options(o) {}
-	ex operator()(const ex & e) { return expand(e, options); }
+	ex operator()(const ex & e) { return e.expand(options); }
 };
 
 /** Expand expression, i.e. multiply it out and return the result as a new
@@ -808,10 +832,16 @@ ex basic::expand(unsigned options) const
  *  1 greater. */
 int basic::compare(const basic & other) const
 {
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_basic_compares++;
+#endif
 	const unsigned hash_this = gethash();
 	const unsigned hash_other = other.gethash();
 	if (hash_this<hash_other) return -1;
 	if (hash_this>hash_other) return 1;
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.compare_same_hashvalue++;
+#endif
 
 	const unsigned typeid_this = tinfo();
 	const unsigned typeid_other = other.tinfo();
@@ -827,6 +857,9 @@ int basic::compare(const basic & other) const
 // 			std::cout << std::endl;
 // 		}
 // 		return cmpval;
+#ifdef GINAC_COMPARE_STATISTICS
+		compare_statistics.compare_same_type++;
+#endif
 		return compare_same_type(other);
 	} else {
 // 		std::cout << "hash collision, different types: " 
@@ -847,13 +880,22 @@ int basic::compare(const basic & other) const
  *  @see is_equal_same_type */
 bool basic::is_equal(const basic & other) const
 {
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_basic_is_equals++;
+#endif
 	if (this->gethash()!=other.gethash())
 		return false;
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.is_equal_same_hashvalue++;
+#endif
 	if (this->tinfo()!=other.tinfo())
 		return false;
 	
 	GINAC_ASSERT(typeid(*this)==typeid(other));
 	
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.is_equal_same_type++;
+#endif
 	return is_equal_same_type(other);
 }
 
@@ -882,4 +924,27 @@ void basic::ensure_if_modifiable() const
 
 int max_recursion_level = 1024;
 
+
+#ifdef GINAC_COMPARE_STATISTICS
+compare_statistics_t::~compare_statistics_t()
+{
+	std::clog << "ex::compare() called " << total_compares << " times" << std::endl;
+	std::clog << "nontrivial compares: " << nontrivial_compares << " times" << std::endl;
+	std::clog << "basic::compare() called " << total_basic_compares << " times" << std::endl;
+	std::clog << "same hashvalue in compare(): " << compare_same_hashvalue << " times" << std::endl;
+	std::clog << "compare_same_type() called " << compare_same_type << " times" << std::endl;
+	std::clog << std::endl;
+	std::clog << "ex::is_equal() called " << total_is_equals << " times" << std::endl;
+	std::clog << "nontrivial is_equals: " << nontrivial_is_equals << " times" << std::endl;
+	std::clog << "basic::is_equal() called " << total_basic_is_equals << " times" << std::endl;
+	std::clog << "same hashvalue in is_equal(): " << is_equal_same_hashvalue << " times" << std::endl;
+	std::clog << "is_equal_same_type() called " << is_equal_same_type << " times" << std::endl;
+	std::clog << std::endl;
+	std::clog << "basic::gethash() called " << total_gethash << " times" << std::endl;
+	std::clog << "used cached hashvalue " << gethash_cached << " times" << std::endl;
+}
+
+compare_statistics_t compare_statistics;
+#endif
+
 } // namespace GiNaC
