@@ -55,14 +55,15 @@ static library_init library_initializer;
 
 class scalar_products;
 class const_iterator;
+class const_preorder_iterator;
+class const_postorder_iterator;
 
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  Basically all it does is
  *  to hold a pointer to the other objects, manage the reference counting and
  *  provide methods for manipulation of these objects.  (Some people call such
  *  a thing a proxy class.) */
-class ex
-{
+class ex {
 	friend class archive_node;
 	friend inline bool are_ex_trivially_equal(const ex &, const ex &);
 	template<class T> friend inline const T &ex_to(const ex &);
@@ -106,12 +107,17 @@ public:
 	// iterators
 	const_iterator begin() const throw();
 	const_iterator end() const throw();
+	const_preorder_iterator preorder_begin() const;
+	const_preorder_iterator preorder_end() const throw();
+	const_postorder_iterator postorder_begin() const;
+	const_postorder_iterator postorder_end() const throw();
 
 	// evaluation
 	ex eval(int level = 0) const { return bp->eval(level); }
 	ex evalf(int level = 0) const { return bp->evalf(level); }
 	ex evalm() const { return bp->evalm(); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
+	ex eval_integ() const { return bp->eval_integ(); }
 
 	// printing
 	void print(const print_context & c, unsigned level = 0) const;
@@ -187,6 +193,7 @@ public:
 	numeric integer_content() const;
 	ex primpart(const ex &x) const;
 	ex primpart(const ex &x, const ex &cont) const;
+	void unitcontprim(const ex &x, ex &u, ex &c, ex &p) const;
 	ex smod(const numeric &xi) const { return bp->smod(xi); }
 	numeric max_coefficient() const;
 
@@ -358,31 +365,53 @@ ex::ex(const std::string &s, const ex &l) : bp(construct_from_string_and_lst(s,
 inline
 int ex::compare(const ex & other) const
 {
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_compares++;
+#endif
 	if (bp == other.bp)  // trivial case: both expressions point to same basic
 		return 0;
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.nontrivial_compares++;
+#endif
 	const int cmpval = bp->compare(*other.bp);
+#if 1
 	if (cmpval == 0) {
 		// Expressions point to different, but equal, trees: conserve
 		// memory and make subsequent compare() operations faster by
-		// making both expression point to the same tree.
+		// making both expressions point to the same tree.
 		share(other);
 	}
+#endif
 	return cmpval;
 }
 
 inline
 bool ex::is_equal(const ex & other) const
 {
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.total_is_equals++;
+#endif
 	if (bp == other.bp)  // trivial case: both expressions point to same basic
 		return true;
-	return bp->is_equal(*other.bp);
+#ifdef GINAC_COMPARE_STATISTICS
+	compare_statistics.nontrivial_is_equals++;
+#endif
+    const bool equal = bp->is_equal(*other.bp);
+#if 0
+	if (equal) {
+		// Expressions point to different, but equal, trees: conserve
+		// memory and make subsequent compare() operations faster by
+		// making both expressions point to the same tree.
+		share(other);
+	}
+#endif
+	return equal;
 }
 
 
 // Iterators
 
-class const_iterator : public std::iterator<std::random_access_iterator_tag, ex, ptrdiff_t, const ex *, const ex &>
-{
+class const_iterator : public std::iterator<std::random_access_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
 	friend class ex;
 	friend class const_preorder_iterator;
 	friend class const_postorder_iterator;
@@ -528,27 +557,24 @@ struct _iter_rep {
 
 } // namespace internal
 
-class const_preorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &>
-{
+class const_preorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
 public:
 	const_preorder_iterator() throw() {}
 
-	// Provide implicit conversion from const_iterator, so ex::begin() and
-	// ex::end() can be used to create const_preorder_iterators
-	const_preorder_iterator(const const_iterator & cit)
+	const_preorder_iterator(const ex &e, size_t n)
 	{
-		s.push(internal::_iter_rep(cit.e, cit.i, cit.e.nops()));
+		s.push(internal::_iter_rep(e, 0, n));
 	}
 
 public:
-	ex operator*() const
+	reference operator*() const
 	{
 		return s.top().e;
 	}
 
-	std::auto_ptr<ex> operator->() const
+	pointer operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return &(s.top().e);
 	}
 
 	const_preorder_iterator &operator++()
@@ -566,7 +592,7 @@ public:
 
 	bool operator==(const const_preorder_iterator &other) const throw()
 	{
-		return s.top() == other.s.top();
+		return s == other.s;
 	}
 
 	bool operator!=(const const_preorder_iterator &other) const throw()
@@ -579,8 +605,10 @@ private:
 
 	void increment()
 	{
-		while (s.top().i == s.top().i_end && s.size() > 1) {
+		while (!s.empty() && s.top().i == s.top().i_end) {
 			s.pop();
+			if (s.empty())
+				return;
 			++s.top().i;
 		}
 
@@ -593,31 +621,25 @@ private:
 	}
 };
 
-class const_postorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &>
-{
+class const_postorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
 public:
 	const_postorder_iterator() throw() {}
 
-	// Provide implicit conversion from const_iterator, so ex::begin() and
-	// ex::end() can be used to create const_postorder_iterators
-	const_postorder_iterator(const const_iterator & cit)
+	const_postorder_iterator(const ex &e, size_t n)
 	{
-		size_t n = cit.e.nops();
-		if (cit.i != n) {
-			s.push(internal::_iter_rep(cit.e, cit.i, n));
-			descend();
-		}
+		s.push(internal::_iter_rep(e, 0, n));
+		descend();
 	}
 
 public:
-	ex operator*() const
+	reference operator*() const
 	{
 		return s.top().e;
 	}
 
-	std::auto_ptr<ex> operator->() const
+	pointer operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return &(s.top().e);
 	}
 
 	const_postorder_iterator &operator++()
@@ -659,7 +681,7 @@ private:
 	{
 		if (s.top().i == s.top().i_end)
 			s.pop();
-		if (s.size() > 0) {
+		if (!s.empty()) {
 			++s.top().i;
 			descend();
 		}
@@ -676,6 +698,26 @@ inline const_iterator ex::end() const throw()
 	return const_iterator(*this, nops());
 }
 
+inline const_preorder_iterator ex::preorder_begin() const
+{
+	return const_preorder_iterator(*this, nops());
+}
+
+inline const_preorder_iterator ex::preorder_end() const throw()
+{
+	return const_preorder_iterator();
+}
+
+inline const_postorder_iterator ex::postorder_begin() const
+{
+	return const_postorder_iterator(*this, nops());
+}
+
+inline const_postorder_iterator ex::postorder_end() const throw()
+{
+	return const_postorder_iterator();
+}
+
 
 // utility functions
 
@@ -712,6 +754,9 @@ inline size_t nops(const ex & thisex)
 inline ex expand(const ex & thisex, unsigned options = 0)
 { return thisex.expand(options); }
 
+inline ex conjugate(const ex & thisex)
+{ return thisex.conjugate(); }
+
 inline bool has(const ex & thisex, const ex & pattern)
 { return thisex.has(pattern); }
 
@@ -763,6 +808,9 @@ inline ex evalf(const ex & thisex, int level = 0)
 inline ex evalm(const ex & thisex)
 { return thisex.evalm(); }
 
+inline ex eval_integ(const ex & thisex)
+{ return thisex.eval_integ(); }
+
 inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
 { return thisex.diff(s, nth); }
 
