@@ -3,7 +3,7 @@
  *  Implementation of sequences of expression pairs. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 #include "power.h"
 #include "relational.h"
 #include "wildcard.h"
-#include "print.h"
 #include "archive.h"
+#include "operators.h"
 #include "utils.h"
 
 #if EXPAIRSEQ_USE_HASHTAB
@@ -42,7 +42,10 @@
 namespace GiNaC {
 
 	
-GINAC_IMPLEMENT_REGISTERED_CLASS_NO_CTORS(expairseq, basic)
+GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(expairseq, basic,
+  print_func<print_context>(&expairseq::do_print).
+  print_func<print_tree>(&expairseq::do_print_tree))
+
 
 //////////
 // helper classes
@@ -58,31 +61,23 @@ public:
 };
 
 //////////
-// default ctor, dtor, copy ctor, assignment operator and helpers
+// default constructor
 //////////
 
 // public
 
-expairseq::expairseq(const expairseq &other)
-{
-	copy(other);
-}
-
-const expairseq &expairseq::operator=(const expairseq &other)
-{
-	if (this != &other) {
-		destroy(true);
-		copy(other);
-	}
-	return *this;
-}
+expairseq::expairseq() : inherited(TINFO_expairseq)
+#if EXPAIRSEQ_USE_HASHTAB
+                                                   , hashtabsize(0)
+#endif // EXPAIRSEQ_USE_HASHTAB
+{}
 
 // protected
 
+#if 0
 /** For use by copy ctor and assignment operator. */
 void expairseq::copy(const expairseq &other)
 {
-	inherited::copy(other);
 	seq = other.seq;
 	overall_coeff = other.overall_coeff;
 #if EXPAIRSEQ_USE_HASHTAB
@@ -104,11 +99,10 @@ void expairseq::copy(const expairseq &other)
 	}
 #endif // EXPAIRSEQ_USE_HASHTAB
 }
-
-DEFAULT_DESTROY(expairseq)
+#endif
 
 //////////
-// other ctors
+// other constructors
 //////////
 
 expairseq::expairseq(const ex &lh, const ex &rh) : inherited(TINFO_expairseq)
@@ -131,13 +125,12 @@ expairseq::expairseq(const epvector &v, const ex &oc)
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(epvector *vp, const ex &oc)
+expairseq::expairseq(std::auto_ptr<epvector> vp, const ex &oc)
   : inherited(TINFO_expairseq), overall_coeff(oc)
 {
-	GINAC_ASSERT(vp!=0);
+	GINAC_ASSERT(vp.get()!=0);
 	GINAC_ASSERT(is_a<numeric>(oc));
 	construct_from_epvector(*vp);
-	delete vp;
 	GINAC_ASSERT(is_canonical());
 }
 
@@ -145,7 +138,7 @@ expairseq::expairseq(epvector *vp, const ex &oc)
 // archiving
 //////////
 
-expairseq::expairseq(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 #if EXPAIRSEQ_USE_HASHTAB
 	, hashtabsize(0)
 #endif
@@ -158,7 +151,11 @@ expairseq::expairseq(const archive_node &n, const lst &sym_lst) : inherited(n, s
 		else
 			break;
 	}
+
 	n.find_ex("overall_coeff", overall_coeff, sym_lst);
+
+	canonicalize();
+	GINAC_ASSERT(is_canonical());
 }
 
 void expairseq::archive(archive_node &n) const
@@ -181,93 +178,85 @@ DEFAULT_UNARCHIVE(expairseq)
 
 // public
 
-basic *expairseq::duplicate() const
+void expairseq::do_print(const print_context & c, unsigned level) const
 {
-	return new expairseq(*this);
+	c.s << "[[";
+	printseq(c, ',', precedence(), level);
+	c.s << "]]";
 }
 
-void expairseq::print(const print_context &c, unsigned level) const
+void expairseq::do_print_tree(const print_tree & c, unsigned level) const
 {
-	if (is_of_type(c, print_tree)) {
-
-		unsigned delta_indent = static_cast<const print_tree &>(c).delta_indent;
-
-		c.s << std::string(level, ' ') << class_name()
-		    << std::hex << ", hash=0x" << hashvalue << ", flags=0x" << flags << std::dec
-		    << ", nops=" << nops()
-		    << std::endl;
-		unsigned num = seq.size();
-		for (unsigned i=0; i<num; ++i) {
-			seq[i].rest.print(c, level + delta_indent);
-			seq[i].coeff.print(c, level + delta_indent);
-			if (i != num - 1)
-				c.s << std::string(level + delta_indent, ' ') << "-----" << std::endl;
-		}
-		if (!overall_coeff.is_equal(default_overall_coeff())) {
-			c.s << std::string(level + delta_indent, ' ') << "-----" << std::endl
-			    << std::string(level + delta_indent, ' ') << "overall_coeff" << std::endl;
-			overall_coeff.print(c, level + delta_indent);
-		}
-		c.s << std::string(level + delta_indent,' ') << "=====" << std::endl;
-#if EXPAIRSEQ_USE_HASHTAB
-		c.s << std::string(level + delta_indent,' ')
-		    << "hashtab size " << hashtabsize << std::endl;
-		if (hashtabsize == 0) return;
-#define MAXCOUNT 5
-		unsigned count[MAXCOUNT+1];
-		for (int i=0; i<MAXCOUNT+1; ++i)
-			count[i] = 0;
-		unsigned this_bin_fill;
-		unsigned cum_fill_sq = 0;
-		unsigned cum_fill = 0;
-		for (unsigned i=0; i<hashtabsize; ++i) {
-			this_bin_fill = 0;
-			if (hashtab[i].size() > 0) {
-				c.s << std::string(level + delta_indent, ' ')
-				    << "bin " << i << " with entries ";
-				for (epplist::const_iterator it=hashtab[i].begin();
-				     it!=hashtab[i].end(); ++it) {
-					c.s << *it-seq.begin() << " ";
-					++this_bin_fill;
-				}
-				c.s << std::endl;
-				cum_fill += this_bin_fill;
-				cum_fill_sq += this_bin_fill*this_bin_fill;
-			}
-			if (this_bin_fill<MAXCOUNT)
-				++count[this_bin_fill];
-			else
-				++count[MAXCOUNT];
-		}
-		unsigned fact = 1;
-		double cum_prob = 0;
-		double lambda = (1.0*seq.size()) / hashtabsize;
-		for (int k=0; k<MAXCOUNT; ++k) {
-			if (k>0)
-				fact *= k;
-			double prob = std::pow(lambda,k)/fact * std::exp(-lambda);
-			cum_prob += prob;
-			c.s << std::string(level + delta_indent, ' ') << "bins with " << k << " entries: "
-			    << int(1000.0*count[k]/hashtabsize)/10.0 << "% (expected: "
-			    << int(prob*1000)/10.0 << ")" << std::endl;
-		}
-		c.s << std::string(level + delta_indent, ' ') << "bins with more entries: "
-		    << int(1000.0*count[MAXCOUNT]/hashtabsize)/10.0 << "% (expected: "
-		    << int((1-cum_prob)*1000)/10.0 << ")" << std::endl;
-	
-		c.s << std::string(level + delta_indent, ' ') << "variance: "
-		    << 1.0/hashtabsize*cum_fill_sq-(1.0/hashtabsize*cum_fill)*(1.0/hashtabsize*cum_fill)
-		    << std::endl;
-		c.s << std::string(level + delta_indent, ' ') << "average fill: "
-		    << (1.0*cum_fill)/hashtabsize
-		    << " (should be equal to " << (1.0*seq.size())/hashtabsize << ")" << std::endl;
-#endif // EXPAIRSEQ_USE_HASHTAB
-
-	} else {
-		c.s << "[[";
-		printseq(c, ',', precedence(), level);
-		c.s << "]]";
+	c.s << std::string(level, ' ') << class_name() << " @" << this
+	    << std::hex << ", hash=0x" << hashvalue << ", flags=0x" << flags << std::dec
+	    << ", nops=" << nops()
+	    << std::endl;
+	size_t num = seq.size();
+	for (size_t i=0; i<num; ++i) {
+		seq[i].rest.print(c, level + c.delta_indent);
+		seq[i].coeff.print(c, level + c.delta_indent);
+		if (i != num - 1)
+			c.s << std::string(level + c.delta_indent, ' ') << "-----" << std::endl;
 	}
+	if (!overall_coeff.is_equal(default_overall_coeff())) {
+		c.s << std::string(level + c.delta_indent, ' ') << "-----" << std::endl
+		    << std::string(level + c.delta_indent, ' ') << "overall_coeff" << std::endl;
+		overall_coeff.print(c, level + c.delta_indent);
+	}
+	c.s << std::string(level + c.delta_indent,' ') << "=====" << std::endl;
+#if EXPAIRSEQ_USE_HASHTAB
+	c.s << std::string(level + c.delta_indent,' ')
+	    << "hashtab size " << hashtabsize << std::endl;
+	if (hashtabsize == 0) return;
+#define MAXCOUNT 5
+	unsigned count[MAXCOUNT+1];
+	for (int i=0; i<MAXCOUNT+1; ++i)
+		count[i] = 0;
+	unsigned this_bin_fill;
+	unsigned cum_fill_sq = 0;
+	unsigned cum_fill = 0;
+	for (unsigned i=0; i<hashtabsize; ++i) {
+		this_bin_fill = 0;
+		if (hashtab[i].size() > 0) {
+			c.s << std::string(level + c.delta_indent, ' ')
+			    << "bin " << i << " with entries ";
+			for (epplist::const_iterator it=hashtab[i].begin();
+			     it!=hashtab[i].end(); ++it) {
+				c.s << *it-seq.begin() << " ";
+				++this_bin_fill;
+			}
+			c.s << std::endl;
+			cum_fill += this_bin_fill;
+			cum_fill_sq += this_bin_fill*this_bin_fill;
+		}
+		if (this_bin_fill<MAXCOUNT)
+			++count[this_bin_fill];
+		else
+			++count[MAXCOUNT];
+	}
+	unsigned fact = 1;
+	double cum_prob = 0;
+	double lambda = (1.0*seq.size()) / hashtabsize;
+	for (int k=0; k<MAXCOUNT; ++k) {
+		if (k>0)
+			fact *= k;
+		double prob = std::pow(lambda,k)/fact * std::exp(-lambda);
+		cum_prob += prob;
+		c.s << std::string(level + c.delta_indent, ' ') << "bins with " << k << " entries: "
+		    << int(1000.0*count[k]/hashtabsize)/10.0 << "% (expected: "
+		    << int(prob*1000)/10.0 << ")" << std::endl;
+	}
+	c.s << std::string(level + c.delta_indent, ' ') << "bins with more entries: "
+	    << int(1000.0*count[MAXCOUNT]/hashtabsize)/10.0 << "% (expected: "
+	    << int((1-cum_prob)*1000)/10.0 << ")" << std::endl;
+
+	c.s << std::string(level + c.delta_indent, ' ') << "variance: "
+	    << 1.0/hashtabsize*cum_fill_sq-(1.0/hashtabsize*cum_fill)*(1.0/hashtabsize*cum_fill)
+	    << std::endl;
+	c.s << std::string(level + c.delta_indent, ' ') << "average fill: "
+	    << (1.0*cum_fill)/hashtabsize
+	    << " (should be equal to " << (1.0*seq.size())/hashtabsize << ")" << std::endl;
+#endif // EXPAIRSEQ_USE_HASHTAB
 }
 
 bool expairseq::info(unsigned inf) const
@@ -275,7 +264,7 @@ bool expairseq::info(unsigned inf) const
 	return inherited::info(inf);
 }
 
-unsigned expairseq::nops() const
+size_t expairseq::nops() const
 {
 	if (overall_coeff.is_equal(default_overall_coeff()))
 		return seq.size();
@@ -283,22 +272,17 @@ unsigned expairseq::nops() const
 		return seq.size()+1;
 }
 
-ex expairseq::op(int i) const
+ex expairseq::op(size_t i) const
 {
-	if (unsigned(i)<seq.size())
+	if (i < seq.size())
 		return recombine_pair_to_ex(seq[i]);
 	GINAC_ASSERT(!overall_coeff.is_equal(default_overall_coeff()));
 	return overall_coeff;
 }
 
-ex &expairseq::let_op(int i)
-{
-	throw(std::logic_error("let_op not defined for expairseq and derived classes (add,mul,...)"));
-}
-
 ex expairseq::map(map_function &f) const
 {
-	epvector *v = new epvector;
+	std::auto_ptr<epvector> v(new epvector);
 	v->reserve(seq.size());
 
 	epvector::const_iterator cit = seq.begin(), last = seq.end();
@@ -319,11 +303,47 @@ ex expairseq::eval(int level) const
 	if ((level==1) && (flags &status_flags::evaluated))
 		return *this;
 	
-	epvector *vp = evalchildren(level);
-	if (vp==0)
+	std::auto_ptr<epvector> vp = evalchildren(level);
+	if (vp.get() == 0)
 		return this->hold();
 	
-	return (new expairseq(vp,overall_coeff))->setflag(status_flags::dynallocated | status_flags::evaluated);
+	return (new expairseq(vp, overall_coeff))->setflag(status_flags::dynallocated | status_flags::evaluated);
+}
+
+epvector* conjugateepvector(const epvector&epv)
+{
+	epvector *newepv = 0;
+	for (epvector::const_iterator i=epv.begin(); i!=epv.end(); ++i) {
+		if(newepv) {
+			newepv->push_back(i->conjugate());
+			continue;
+		}
+		expair x = i->conjugate();
+		if (x.is_equal(*i)) {
+			continue;
+		}
+		newepv = new epvector;
+		newepv->reserve(epv.size());
+		for (epvector::const_iterator j=epv.begin(); j!=i; ++j) {
+			newepv->push_back(*j);
+		}
+		newepv->push_back(x);
+	}
+	return newepv;
+}
+
+ex expairseq::conjugate() const
+{
+	epvector* newepv = conjugateepvector(seq);
+	ex x = overall_coeff.conjugate();
+	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
+		return *this;
+	}
+	ex result = thisexpairseq(newepv ? *newepv : seq, x);
+	if (newepv) {
+		delete newepv;
+	}
+	return result;
 }
 
 bool expairseq::match(const ex & pattern, lst & repl_lst) const
@@ -337,8 +357,8 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 		// expression", like "*" above) is present
 		bool has_global_wildcard = false;
 		ex global_wildcard;
-		for (unsigned int i=0; i<pattern.nops(); i++) {
-			if (is_ex_exactly_of_type(pattern.op(i), wildcard)) {
+		for (size_t i=0; i<pattern.nops(); i++) {
+			if (is_exactly_a<wildcard>(pattern.op(i))) {
 				has_global_wildcard = true;
 				global_wildcard = pattern.op(i);
 				break;
@@ -351,12 +371,12 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 		// Chop into terms
 		exvector ops;
 		ops.reserve(nops());
-		for (unsigned i=0; i<nops(); i++)
+		for (size_t i=0; i<nops(); i++)
 			ops.push_back(op(i));
 
 		// Now, for every term of the pattern, look for a matching term in
 		// the expression and remove the match
-		for (unsigned i=0; i<pattern.nops(); i++) {
+		for (size_t i=0; i<pattern.nops(); i++) {
 			ex p = pattern.op(i);
 			if (has_global_wildcard && p.is_equal(global_wildcard))
 				continue;
@@ -377,15 +397,15 @@ found:		;
 			// Assign all the remaining terms to the global wildcard (unless
 			// it has already been matched before, in which case the matches
 			// must be equal)
-			unsigned num = ops.size();
-			epvector *vp = new epvector();
+			size_t num = ops.size();
+			std::auto_ptr<epvector> vp(new epvector);
 			vp->reserve(num);
-			for (unsigned i=0; i<num; i++)
+			for (size_t i=0; i<num; i++)
 				vp->push_back(split_ex_to_pair(ops[i]));
 			ex rest = thisexpairseq(vp, default_overall_coeff());
-			for (unsigned i=0; i<repl_lst.nops(); i++) {
-				if (repl_lst.op(i).op(0).is_equal(global_wildcard))
-					return rest.is_equal(repl_lst.op(i).op(1));
+			for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it) {
+				if (it->op(0).is_equal(global_wildcard))
+					return rest.is_equal(it->op(1));
 			}
 			repl_lst.append(global_wildcard == rest);
 			return true;
@@ -400,13 +420,15 @@ found:		;
 	return inherited::match(pattern, repl_lst);
 }
 
-ex expairseq::subs(const lst &ls, const lst &lr, bool no_pattern) const
+ex expairseq::subs(const exmap & m, unsigned options) const
 {
-	epvector *vp = subschildren(ls, lr, no_pattern);
-	if (vp)
+	std::auto_ptr<epvector> vp = subschildren(m, options);
+	if (vp.get())
 		return ex_to<basic>(thisexpairseq(vp, overall_coeff));
+	else if ((options & subs_options::algebraic) && is_exactly_a<mul>(*this))
+		return static_cast<const mul *>(this)->algebraic_subs_mul(m, options);
 	else
-		return basic::subs(ls, lr, no_pattern);
+		return subs_one_level(m, options);
 }
 
 // protected
@@ -541,30 +563,28 @@ bool expairseq::is_equal_same_type(const basic &other) const
 #endif // EXPAIRSEQ_USE_HASHTAB
 }
 
-unsigned expairseq::return_type(void) const
+unsigned expairseq::return_type() const
 {
 	return return_types::noncommutative_composite;
 }
 
-unsigned expairseq::calchash(void) const
+unsigned expairseq::calchash() const
 {
 	unsigned v = golden_ratio_hash(this->tinfo());
-	epvector::const_iterator i = seq.begin(), end = seq.end();
+	epvector::const_iterator i = seq.begin();
+	const epvector::const_iterator end = seq.end();
 	while (i != end) {
-#if !EXPAIRSEQ_USE_HASHTAB
-		v = rotate_left_31(v); // rotation would spoil commutativity
-#endif // EXPAIRSEQ_USE_HASHTAB
 		v ^= i->rest.gethash();
 #if !EXPAIRSEQ_USE_HASHTAB
-		v = rotate_left_31(v);
+		// rotation spoils commutativity!
+		v = rotate_left(v);
 		v ^= i->coeff.gethash();
-#endif // EXPAIRSEQ_USE_HASHTAB
+#endif // !EXPAIRSEQ_USE_HASHTAB
 		++i;
 	}
-	
+
 	v ^= overall_coeff.gethash();
-	v &= 0x7FFFFFFFU;
-	
+
 	// store calculated hash value only if object is already evaluated
 	if (flags &status_flags::evaluated) {
 		setflag(status_flags::hash_calculated);
@@ -576,12 +596,13 @@ unsigned expairseq::calchash(void) const
 
 ex expairseq::expand(unsigned options) const
 {
-	epvector *vp = expandchildren(options);
-	if (vp == NULL) {
+	std::auto_ptr<epvector> vp = expandchildren(options);
+	if (vp.get())
+		return thisexpairseq(vp, overall_coeff);
+	else {
 		// The terms have not changed, so it is safe to declare this expanded
 		return (options == 0) ? setflag(status_flags::expanded) : *this;
-	} else
-		return thisexpairseq(vp, overall_coeff);
+	}
 }
 
 //////////
@@ -600,12 +621,12 @@ ex expairseq::expand(unsigned options) const
  *  definition. */
 ex expairseq::thisexpairseq(const epvector &v, const ex &oc) const
 {
-	return expairseq(v,oc);
+	return expairseq(v, oc);
 }
 
-ex expairseq::thisexpairseq(epvector *vp, const ex &oc) const
+ex expairseq::thisexpairseq(std::auto_ptr<epvector> vp, const ex &oc) const
 {
-	return expairseq(vp,oc);
+	return expairseq(vp, oc);
 }
 
 void expairseq::printpair(const print_context & c, const expair & p, unsigned upper_precedence) const
@@ -681,7 +702,7 @@ bool expairseq::expair_needs_further_processing(epp it)
 	return false;
 }
 
-ex expairseq::default_overall_coeff(void) const
+ex expairseq::default_overall_coeff() const
 {
 	return _ex0;
 }
@@ -777,8 +798,8 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 	hashtabsize = 0;
 #endif // EXPAIRSEQ_USE_HASHTAB
 	
-	if (is_ex_exactly_of_type(lh,numeric)) {
-		if (is_ex_exactly_of_type(rh,numeric)) {
+	if (is_exactly_a<numeric>(lh)) {
+		if (is_exactly_a<numeric>(rh)) {
 			combine_overall_coeff(lh);
 			combine_overall_coeff(rh);
 		} else {
@@ -786,7 +807,7 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 			seq.push_back(split_ex_to_pair(rh));
 		}
 	} else {
-		if (is_ex_exactly_of_type(rh,numeric)) {
+		if (is_exactly_a<numeric>(rh)) {
 			combine_overall_coeff(rh);
 			seq.push_back(split_ex_to_pair(lh));
 		} else {
@@ -873,7 +894,7 @@ void expairseq::construct_from_expairseq_ex(const expairseq &s,
 											const ex &e)
 {
 	combine_overall_coeff(s.overall_coeff);
-	if (is_ex_exactly_of_type(e,numeric)) {
+	if (is_exactly_a<numeric>(e)) {
 		combine_overall_coeff(e);
 		seq = s.seq;
 		return;
@@ -998,7 +1019,7 @@ void expairseq::make_flat(const exvector &v)
 				++cit_s;
 			}
 		} else {
-			if (is_ex_exactly_of_type(*cit,numeric))
+			if (is_exactly_a<numeric>(*cit))
 				combine_overall_coeff(*cit);
 			else
 				seq.push_back(split_ex_to_pair(*cit));
@@ -1057,7 +1078,7 @@ void expairseq::make_flat(const epvector &v)
 }
 
 /** Brings this expairseq into a sorted (canonical) form. */
-void expairseq::canonicalize(void)
+void expairseq::canonicalize()
 {
 	std::sort(seq.begin(), seq.end(), expair_rest_is_less());
 }
@@ -1066,7 +1087,7 @@ void expairseq::canonicalize(void)
 /** Compact a presorted expairseq by combining all matching expairs to one
  *  each.  On an add object, this is responsible for 2*x+3*x+y -> 5*x+y, for
  *  instance. */
-void expairseq::combine_same_terms_sorted_seq(void)
+void expairseq::combine_same_terms_sorted_seq()
 {
 	if (seq.size()<2)
 		return;
@@ -1123,7 +1144,7 @@ unsigned expairseq::calc_hashtabsize(unsigned sz) const
 	size = nearest_power_of_2/hashtabfactor;
 	if (size<minhashtabsize)
 		return 0;
-	GINAC_ASSERT(hashtabsize<=0x8000000U); // really max size due to 31 bit hashing
+
 	// hashtabsize must be a power of 2
 	GINAC_ASSERT((1U << log2(size))==size);
 	return size;
@@ -1132,21 +1153,19 @@ unsigned expairseq::calc_hashtabsize(unsigned sz) const
 unsigned expairseq::calc_hashindex(const ex &e) const
 {
 	// calculate hashindex
-	unsigned hash = e.gethash();
 	unsigned hashindex;
-	if (is_a_numeric_hash(hash)) {
+	if (is_a<numeric>(e)) {
 		hashindex = hashmask;
 	} else {
-		hashindex = hash &hashmask;
+		hashindex = e.gethash() & hashmask;
 		// last hashtab entry is reserved for numerics
 		if (hashindex==hashmask) hashindex = 0;
 	}
-	GINAC_ASSERT(hashindex>=0);
 	GINAC_ASSERT((hashindex<hashtabsize)||(hashtabsize==0));
 	return hashindex;
 }
 
-void expairseq::shrink_hashtab(void)
+void expairseq::shrink_hashtab()
 {
 	unsigned new_hashtabsize;
 	while (hashtabsize!=(new_hashtabsize=calc_hashtabsize(seq.size()))) {
@@ -1250,7 +1269,7 @@ void expairseq::build_hashtab_and_combine(epvector::iterator &first_numeric,
 	epp current = seq.begin();
 
 	while (current!=first_numeric) {
-		if (is_ex_exactly_of_type(current->rest,numeric)) {
+		if (is_exactly_a<numeric>(current->rest)) {
 			--first_numeric;
 			iter_swap(current,first_numeric);
 		} else {
@@ -1296,7 +1315,7 @@ void expairseq::drop_coeff_0_terms(epvector::iterator &first_numeric,
 	// move terms with coeff 0 to end and remove them from hashtab
 	// check only those elements which have been touched
 	epp current = seq.begin();
-	unsigned i = 0;
+	size_t i = 0;
 	while (current!=first_numeric) {
 		if (!touched[i]) {
 			++current;
@@ -1343,7 +1362,7 @@ void expairseq::drop_coeff_0_terms(epvector::iterator &first_numeric,
 /** True if one of the coeffs vanishes, otherwise false.
  *  This would be an invariant violation, so this should only be used for
  *  debugging purposes. */
-bool expairseq::has_coeff_0(void) const
+bool expairseq::has_coeff_0() const
 {
 	epvector::const_iterator i = seq.begin(), end = seq.end();
 	while (i != end) {
@@ -1366,7 +1385,7 @@ void expairseq::add_numerics_to_hashtab(epvector::iterator first_numeric,
 	}
 }
 
-void expairseq::combine_same_terms(void)
+void expairseq::combine_same_terms()
 {
 	// combine same terms, drop term with coeff 0, move numerics to end
 	
@@ -1392,7 +1411,7 @@ void expairseq::combine_same_terms(void)
 	epvector::iterator first_numeric = seq.end();
 	epvector::iterator last_non_zero = seq.end()-1;
 	
-	unsigned num = seq.size();
+	size_t num = seq.size();
 	std::vector<bool> touched(num);
 	
 	unsigned number_of_zeroes = 0;
@@ -1438,11 +1457,11 @@ bool expairseq::is_canonical() const
 	epvector::const_iterator it_last = it;
 	for (++it; it!=itend; it_last=it, ++it) {
 		if (!(it_last->is_less(*it) || it_last->is_equal(*it))) {
-			if (!is_ex_exactly_of_type(it_last->rest,numeric) ||
-				!is_ex_exactly_of_type(it->rest,numeric)) {
+			if (!is_exactly_a<numeric>(it_last->rest) ||
+				!is_exactly_a<numeric>(it->rest)) {
 				// double test makes it easier to set a breakpoint...
-				if (!is_ex_exactly_of_type(it_last->rest,numeric) ||
-					!is_ex_exactly_of_type(it->rest,numeric)) {
+				if (!is_exactly_a<numeric>(it_last->rest) ||
+					!is_exactly_a<numeric>(it->rest)) {
 					printpair(std::clog, *it_last, 0);
 					std::clog << ">";
 					printpair(std::clog, *it, 0);
@@ -1467,7 +1486,7 @@ bool expairseq::is_canonical() const
  *  @see expairseq::expand()
  *  @return pointer to epvector containing expanded pairs or zero pointer,
  *  if no members were changed. */
-epvector * expairseq::expandchildren(unsigned options) const
+std::auto_ptr<epvector> expairseq::expandchildren(unsigned options) const
 {
 	const epvector::const_iterator last = seq.end();
 	epvector::const_iterator cit = seq.begin();
@@ -1476,7 +1495,7 @@ epvector * expairseq::expandchildren(unsigned options) const
 		if (!are_ex_trivially_equal(cit->rest,expanded_ex)) {
 			
 			// something changed, copy seq, eval and return it
-			epvector *s = new epvector;
+			std::auto_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
@@ -1485,10 +1504,12 @@ epvector * expairseq::expandchildren(unsigned options) const
 				s->push_back(*cit2);
 				++cit2;
 			}
+
 			// copy first changed element
 			s->push_back(combine_ex_with_coeff_to_pair(expanded_ex,
 			                                           cit2->coeff));
 			++cit2;
+
 			// copy rest
 			while (cit2!=last) {
 				s->push_back(combine_ex_with_coeff_to_pair(cit2->rest.expand(options),
@@ -1500,7 +1521,7 @@ epvector * expairseq::expandchildren(unsigned options) const
 		++cit;
 	}
 	
-	return 0; // signalling nothing has changed
+	return std::auto_ptr<epvector>(0); // signalling nothing has changed
 }
 
 
@@ -1509,14 +1530,14 @@ epvector * expairseq::expandchildren(unsigned options) const
  *  @see expairseq::eval()
  *  @return pointer to epvector containing evaluated pairs or zero pointer,
  *  if no members were changed. */
-epvector * expairseq::evalchildren(int level) const
+std::auto_ptr<epvector> expairseq::evalchildren(int level) const
 {
 	// returns a NULL pointer if nothing had to be evaluated
 	// returns a pointer to a newly created epvector otherwise
 	// (which has to be deleted somewhere else)
 
 	if (level==1)
-		return 0;
+		return std::auto_ptr<epvector>(0);
 	
 	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
@@ -1529,7 +1550,7 @@ epvector * expairseq::evalchildren(int level) const
 		if (!are_ex_trivially_equal(cit->rest,evaled_ex)) {
 			
 			// something changed, copy seq, eval and return it
-			epvector *s = new epvector;
+			std::auto_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
@@ -1538,10 +1559,12 @@ epvector * expairseq::evalchildren(int level) const
 				s->push_back(*cit2);
 				++cit2;
 			}
+
 			// copy first changed element
 			s->push_back(combine_ex_with_coeff_to_pair(evaled_ex,
 			                                           cit2->coeff));
 			++cit2;
+
 			// copy rest
 			while (cit2!=last) {
 				s->push_back(combine_ex_with_coeff_to_pair(cit2->rest.eval(level),
@@ -1553,7 +1576,7 @@ epvector * expairseq::evalchildren(int level) const
 		++cit;
 	}
 	
-	return 0; // signalling nothing has changed
+	return std::auto_ptr<epvector>(0); // signalling nothing has changed
 }
 
 
@@ -1562,32 +1585,36 @@ epvector * expairseq::evalchildren(int level) const
  *  @see expairseq::subs()
  *  @return pointer to epvector containing pairs after application of subs,
  *    or NULL pointer if no members were changed. */
-epvector * expairseq::subschildren(const lst &ls, const lst &lr, bool no_pattern) const
+std::auto_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned options) const
 {
-	GINAC_ASSERT(ls.nops()==lr.nops());
+	// When any of the objects to be substituted is a product or power
+	// we have to recombine the pairs because the numeric coefficients may
+	// be part of the search pattern.
+	if (!(options & (subs_options::pattern_is_product | subs_options::pattern_is_not_product))) {
 
-	// The substitution is "complex" when any of the objects to be substituted
-	// is a product or power. In this case we have to recombine the pairs
-	// because the numeric coefficients may be part of the search pattern.
-	bool complex_subs = false;
-	for (unsigned i=0; i<ls.nops(); ++i)
-		if (is_ex_exactly_of_type(ls.op(i), mul) || is_ex_exactly_of_type(ls.op(i), power)) {
-			complex_subs = true;
-			break;
+		// Search the list of substitutions and cache our findings
+		for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
+			if (is_exactly_a<mul>(it->first) || is_exactly_a<power>(it->first)) {
+				options |= subs_options::pattern_is_product;
+				break;
+			}
 		}
+		if (!(options & subs_options::pattern_is_product))
+			options |= subs_options::pattern_is_not_product;
+	}
 
-	if (complex_subs) {
+	if (options & subs_options::pattern_is_product) {
 
 		// Substitute in the recombined pairs
 		epvector::const_iterator cit = seq.begin(), last = seq.end();
 		while (cit != last) {
 
 			const ex &orig_ex = recombine_pair_to_ex(*cit);
-			const ex &subsed_ex = orig_ex.subs(ls, lr, no_pattern);
+			const ex &subsed_ex = orig_ex.subs(m, options);
 			if (!are_ex_trivially_equal(orig_ex, subsed_ex)) {
 
 				// Something changed, copy seq, subs and return it
-				epvector *s = new epvector;
+				std::auto_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
 				// Copy parts of seq which are known not to have changed
@@ -1599,7 +1626,7 @@ epvector * expairseq::subschildren(const lst &ls, const lst &lr, bool no_pattern
 
 				// Copy rest
 				while (cit != last) {
-					s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit).subs(ls, lr, no_pattern)));
+					s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit).subs(m, options)));
 					++cit;
 				}
 				return s;
@@ -1614,11 +1641,11 @@ epvector * expairseq::subschildren(const lst &ls, const lst &lr, bool no_pattern
 		epvector::const_iterator cit = seq.begin(), last = seq.end();
 		while (cit != last) {
 
-			const ex &subsed_ex = cit->rest.subs(ls, lr, no_pattern);
+			const ex &subsed_ex = cit->rest.subs(m, options);
 			if (!are_ex_trivially_equal(cit->rest, subsed_ex)) {
 			
 				// Something changed, copy seq, subs and return it
-				epvector *s = new epvector;
+				std::auto_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
 				// Copy parts of seq which are known not to have changed
@@ -1630,7 +1657,7 @@ epvector * expairseq::subschildren(const lst &ls, const lst &lr, bool no_pattern
 
 				// Copy rest
 				while (cit != last) {
-					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(ls, lr, no_pattern),
+					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options),
 					                                           cit->coeff));
 					++cit;
 				}
@@ -1642,7 +1669,7 @@ epvector * expairseq::subschildren(const lst &ls, const lst &lr, bool no_pattern
 	}
 	
 	// Nothing has changed
-	return NULL;
+	return std::auto_ptr<epvector>(0);
 }
 
 //////////
