@@ -4,7 +4,7 @@
  *  somewhat obsolete (most of this can be replaced by exceptions). */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,19 +24,20 @@
 #include <iostream>
 
 #include "fail.h"
-#include "print.h"
 #include "archive.h"
 #include "utils.h"
 
 namespace GiNaC {
 
-GINAC_IMPLEMENT_REGISTERED_CLASS(fail, basic)
+GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fail, basic,
+  print_func<print_context>(&fail::do_print).
+  print_func<print_tree>(&fail::do_print_tree))
 
 //////////
-// default ctor, dtor, copy ctor, assignment operator and helpers
+// default constructor
 //////////
 
-DEFAULT_CTORS(fail)
+DEFAULT_CTOR(fail)
 
 //////////
 // archiving
