@@ -36,6 +36,7 @@
 #include "lst.h"
 #include "archive.h"
 #include "utils.h"
+#include "integral.h"
 
 namespace GiNaC {
 
@@ -48,7 +49,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(indexed, exprseq,
 // default constructor
 //////////
 
-indexed::indexed() : symtree(sy_none())
+indexed::indexed() : symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 }
@@ -57,31 +58,31 @@ indexed::indexed() : symtree(sy_none())
 // other constructors
 //////////
 
-indexed::indexed(const ex & b) : inherited(b), symtree(sy_none())
+indexed::indexed(const ex & b) : inherited(b), symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1) : inherited(b, i1), symtree(sy_none())
+indexed::indexed(const ex & b, const ex & i1) : inherited(b, i1), symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(sy_none())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2) : inherited(b, i1, i2), symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(sy_none())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3) : inherited(b, i1, i2, i3), symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 	validate();
 }
 
-indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(sy_none())
+indexed::indexed(const ex & b, const ex & i1, const ex & i2, const ex & i3, const ex & i4) : inherited(b, i1, i2, i3, i4), symtree(not_symmetric())
 {
 	tinfo_key = TINFO_indexed;
 	validate();
@@ -105,7 +106,7 @@ indexed::indexed(const ex & b, const symmetry & symm, const ex & i1, const ex &
 	validate();
 }
 
-indexed::indexed(const ex & b, const exvector & v) : inherited(b), symtree(sy_none())
+indexed::indexed(const ex & b, const exvector & v) : inherited(b), symtree(not_symmetric())
 {
 	seq.insert(seq.end(), v.begin(), v.end());
 	tinfo_key = TINFO_indexed;
@@ -152,7 +153,7 @@ indexed::indexed(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 				symtree = sy_anti();
 				break;
 			default:
-				symtree = sy_none();
+				symtree = not_symmetric();
 				break;
 		}
 		const_cast<symmetry &>(ex_to<symmetry>(symtree)).validate(seq.size() - 1);
@@ -499,10 +500,34 @@ exvector ncmul::get_free_indices() const
 	return free_indices;
 }
 
+struct is_summation_idx : public std::unary_function<ex, bool> {
+	bool operator()(const ex & e)
+	{
+		return is_dummy_pair(e, e);
+	}
+};
+
 exvector power::get_free_indices() const
 {
-	// Return free indices of basis
-	return basis.get_free_indices();
+	// Get free indices of basis
+	exvector basis_indices = basis.get_free_indices();
+
+	if (exponent.info(info_flags::even)) {
+		// If the exponent is an even number, then any "free" index that
+		// forms a dummy pair with itself is actually a summation index
+		exvector really_free;
+		std::remove_copy_if(basis_indices.begin(), basis_indices.end(),
+		                    std::back_inserter(really_free), is_summation_idx());
+		return really_free;
+	} else
+		return basis_indices;
+}
+
+exvector integral::get_free_indices() const
+{
+	if (a.get_free_indices().size() || b.get_free_indices().size())
+		throw (std::runtime_error("integral::get_free_indices: boundary values should not have free indices"));
+	return f.get_free_indices();
 }
 
 /** Rename dummy indices in an expression.
@@ -1111,6 +1136,7 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
  *  performs contraction of dummy indices where possible and checks whether
  *  the free indices in sums are consistent.
  *
+ *  @param options Simplification options (currently unused)
  *  @return simplified expression */
 ex ex::simplify_indexed(unsigned options) const
 {
@@ -1125,6 +1151,7 @@ ex ex::simplify_indexed(unsigned options) const
  *  scalar products by known values if desired.
  *
  *  @param sp Scalar products to be replaced automatically
+ *  @param options Simplification options (currently unused)
  *  @return simplified expression */
 ex ex::simplify_indexed(const scalar_products & sp, unsigned options) const
 {
