@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 29 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 29 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
 
 #include <stdexcept>
 
@@ -146,22 +146,29 @@ typedef int YYSTYPE;
 
 
 /* Line 214 of yacc.c.  */
-#line 149 "input_parser.cc"
+#line 150 "input_parser.cc"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# if YYSTACK_USE_ALLOCA
-#  define YYSTACK_ALLOC alloca
+# ifdef YYSTACK_USE_ALLOCA
+#  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
 # else
-#  ifndef YYSTACK_USE_ALLOCA
-#   if defined (alloca) || defined (_ALLOCA_H)
-#    define YYSTACK_ALLOC alloca
-#   else
-#    ifdef __GNUC__
-#     define YYSTACK_ALLOC __builtin_alloca
-#    endif
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
+#   ifdef __GNUC__
+#    define YYSTACK_ALLOC __builtin_alloca
 #   endif
 #  endif
 # endif
@@ -174,15 +181,15 @@ typedef int YYSTYPE;
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC malloc
-#  define YYSTACK_FREE free
+#  define YYSTACK_ALLOC YYMALLOC
+#  define YYSTACK_FREE YYFREE
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYSTYPE_IS_TRIVIAL)))
+	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
@@ -203,7 +210,7 @@ union yyalloc
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if 1 < __GNUC__
+#  if defined (__GNUC__) && 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -339,12 +346,12 @@ static const unsigned char yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "T_EOF", "T_NUMBER", "T_SYMBOL", 
-  "T_LITERAL", "T_DIGITS", "T_EQUAL", "T_NOTEQ", "T_LESSEQ", 
-  "T_GREATEREQ", "'='", "'<'", "'>'", "'+'", "'-'", "'*'", "'/'", "'%'", 
-  "NEG", "'^'", "'.'", "'~'", "'!'", "'('", "')'", "'{'", "'}'", "'['", 
-  "']'", "','", "$accept", "input", "exp", "exprseq", "list_or_empty", 
-  "list", "matrix", "row", 0
+  "$end", "error", "$undefined", "T_EOF", "T_NUMBER", "T_SYMBOL",
+  "T_LITERAL", "T_DIGITS", "T_EQUAL", "T_NOTEQ", "T_LESSEQ", "T_GREATEREQ",
+  "'='", "'<'", "'>'", "'+'", "'-'", "'*'", "'/'", "'%'", "NEG", "'^'",
+  "'.'", "'~'", "'!'", "'('", "')'", "'{'", "'}'", "'['", "']'", "','",
+  "$accept", "input", "exp", "exprseq", "list_or_empty", "list", "matrix",
+  "row", 0
 };
 #endif
 
@@ -495,7 +502,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrlab1
+#define YYERROR		goto yyerrorlab
 
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
@@ -530,11 +537,11 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)         \
-  Current.first_line   = Rhs[1].first_line;      \
-  Current.first_column = Rhs[1].first_column;    \
-  Current.last_line    = Rhs[N].last_line;       \
-  Current.last_column  = Rhs[N].last_column;
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
@@ -578,7 +585,7 @@ do {								\
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (cinluded).                                                   |
+| TOP (included).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
@@ -618,9 +625,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylineno = yyrline[yyrule];
+  unsigned int yylno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylineno);
+             yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -657,7 +664,7 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
-#if YYMAXDEPTH == 0
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
 # undef YYMAXDEPTH
 #endif
 
@@ -1080,7 +1087,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 82 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 82 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {
 		try {
 			parsed_ex = yyvsp[-1];
@@ -1093,12 +1100,12 @@ yyreduce:
     break;
 
   case 3:
-#line 93 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 93 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 4:
-#line 94 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 94 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {
 		if (is_lexer_symbol_predefined(yyvsp[0]))
 			yyval = yyvsp[0].eval();
@@ -1108,17 +1115,17 @@ yyreduce:
     break;
 
   case 5:
-#line 100 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 100 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 6:
-#line 101 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 101 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 7:
-#line 102 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 102 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {
 		std::string n = get_symbol_name(yyvsp[-3]);
 		if (n == "sqrt") {
@@ -1133,155 +1140,155 @@ yyreduce:
     break;
 
   case 8:
-#line 113 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 113 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] == yyvsp[0];}
     break;
 
   case 9:
-#line 114 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 114 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] != yyvsp[0];}
     break;
 
   case 10:
-#line 115 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 115 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] < yyvsp[0];}
     break;
 
   case 11:
-#line 116 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 116 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] <= yyvsp[0];}
     break;
 
   case 12:
-#line 117 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 117 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] > yyvsp[0];}
     break;
 
   case 13:
-#line 118 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 118 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] >= yyvsp[0];}
     break;
 
   case 14:
-#line 119 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 119 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] + yyvsp[0];}
     break;
 
   case 15:
-#line 120 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 120 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] - yyvsp[0];}
     break;
 
   case 16:
-#line 121 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 121 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] * yyvsp[0];}
     break;
 
   case 17:
-#line 122 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 122 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-2] / yyvsp[0];}
     break;
 
   case 18:
-#line 123 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 123 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = -yyvsp[0];}
     break;
 
   case 19:
-#line 124 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 124 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 20:
-#line 125 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 125 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = pow(yyvsp[-2], yyvsp[0]);}
     break;
 
   case 21:
-#line 126 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 126 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = attach_index(yyvsp[-2], yyvsp[0], true);}
     break;
 
   case 22:
-#line 127 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 127 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = attach_index(yyvsp[-2], yyvsp[0], false);}
     break;
 
   case 23:
-#line 128 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 128 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = factorial(yyvsp[-1]);}
     break;
 
   case 24:
-#line 129 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 129 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-1];}
     break;
 
   case 25:
-#line 130 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 130 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[-1];}
     break;
 
   case 26:
-#line 131 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 131 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = lst_to_matrix(ex_to<lst>(yyvsp[-1]));}
     break;
 
   case 27:
-#line 134 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 134 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = exprseq(yyvsp[0]);}
     break;
 
   case 28:
-#line 135 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 135 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {exprseq es(ex_to<exprseq>(yyvsp[-2])); yyval = es.append(yyvsp[0]);}
     break;
 
   case 29:
-#line 138 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 138 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = *new lst;}
     break;
 
   case 30:
-#line 139 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 139 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 31:
-#line 142 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 142 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = lst(yyvsp[0]);}
     break;
 
   case 32:
-#line 143 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 143 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 
   case 33:
-#line 146 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 146 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = lst(yyvsp[-1]);}
     break;
 
   case 34:
-#line 147 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 147 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-4])); yyval = l.append(yyvsp[-1]);}
     break;
 
   case 35:
-#line 150 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 150 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {yyval = lst(yyvsp[0]);}
     break;
 
   case 36:
-#line 151 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 151 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 
 
     }
 
-/* Line 999 of yacc.c.  */
-#line 1284 "input_parser.cc"
+/* Line 1000 of yacc.c.  */
+#line 1292 "input_parser.cc"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1322,18 +1329,33 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
+	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx, yycount;
+	  int yyx;
 
-	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  for (yyx = yyn < 0 ? -yyn : 0;
-	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
+	  int yyxbegin = yyn < 0 ? -yyn : 0;
+
+	  /* Stay within bounds of both yycheck and yytname.  */
+	  int yychecklim = YYLAST - yyn;
+	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+	  int yycount = 0;
+
+	  yyprefix = ", expecting ";
+	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("syntax error, unexpected ") + 1;
-	  yysize += yystrlen (yytname[yytype]);
+	      {
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
+		  {
+		    yysize = 0;
+		    break;
+		  }
+	      }
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1342,16 +1364,13 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yycount = 0;
-		  for (yyx = yyn < 0 ? -yyn : 0;
-		       yyx < (int) (sizeof (yytname) / sizeof (char *));
-		       yyx++)
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			const char *yyq = ! yycount ? ", expecting " : " or ";
-			yyp = yystpcpy (yyp, yyq);
+			yyp = yystpcpy (yyp, yyprefix);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yycount++;
+			yyprefix = " or ";
 		      }
 		}
 	      yyerror (yymsg);
@@ -1372,25 +1391,27 @@ yyerrlab:
       /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      /* Return failure if at end of input.  */
-      if (yychar == YYEOF)
+      if (yychar <= YYEOF)
         {
-	  /* Pop the error token.  */
-          YYPOPSTACK;
-	  /* Pop the rest of the stack.  */
-	  while (yyss < yyssp)
-	    {
-	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
-	      yydestruct (yystos[*yyssp], yyvsp);
-	      YYPOPSTACK;
-	    }
-	  YYABORT;
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
+	  if (yychar == YYEOF)
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
+      else
+	{
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
+	  yychar = YYEMPTY;
 
-      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
-      yydestruct (yytoken, &yylval);
-      yychar = YYEMPTY;
-
+	}
     }
 
   /* Else will try to reuse lookahead token after shifting the error
@@ -1398,9 +1419,27 @@ yyerrlab:
   goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
+/*---------------------------------------------------.
+| yyerrorlab -- error raised explicitly by YYERROR.  |
+`---------------------------------------------------*/
+yyerrorlab:
+
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
+  if (0)
+     goto yyerrorlab;
+#endif
+
+  yyvsp -= yylen;
+  yyssp -= yylen;
+  yystate = *yyssp;
+  goto yyerrlab1;
+
+
+/*-------------------------------------------------------------.
+| yyerrlab1 -- common code for both syntax error and YYERROR.  |
+`-------------------------------------------------------------*/
 yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
@@ -1424,9 +1463,8 @@ yyerrlab1:
 
       YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
       yydestruct (yystos[yystate], yyvsp);
-      yyvsp--;
-      yystate = *--yyssp;
-
+      YYPOPSTACK;
+      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
@@ -1475,7 +1513,7 @@ yyreturn:
 }
 
 
-#line 159 "/home/cbauer/projects/GiNaC-1.2/ginac/input_parser.yy"
+#line 159 "/home/cbauer/projects/GiNaC-HEAD/ginac/input_parser.yy"
 
 // Attach index to expression
 ex attach_index(const ex & base, ex i, bool covariant)
