@@ -32,16 +32,16 @@ namespace GiNaC {
 typedef container<std::list> lst;
 
 /** Specialization of container::get_tinfo() for lst. */
-inline unsigned lst::get_tinfo() { return TINFO_lst; }
+template<> inline unsigned lst::get_tinfo() { return TINFO_lst; }
 
 /** Specialization of container::get_default_flags() for lst. */
-inline unsigned lst::get_default_flags() { return status_flags::not_shareable; }
+template<> inline unsigned lst::get_default_flags() { return status_flags::not_shareable; }
 
 /** Specialization of container::get_open_delim() for lst. */
-inline char lst::get_open_delim() { return '{'; }
+template<> inline char lst::get_open_delim() { return '{'; }
 
 /** Specialization of container::get_close_delim() for lst. */
-inline char lst::get_close_delim() { return '}'; }
+template<> inline char lst::get_close_delim() { return '}'; }
 
 // defined in lst.cpp
 template<> bool lst::info(unsigned inf) const;
