@@ -1628,8 +1628,8 @@ const numeric binomial(const numeric &n, const numeric &k)
 		}
 	}
 	
-	// should really be gamma(n+1)/gamma(r+1)/gamma(n-r+1) or a suitable limit
-	throw std::range_error("numeric::binomial(): dont know how to evaluate that.");
+	// should really be gamma(n+1)/gamma(k+1)/gamma(n-k+1) or a suitable limit
+	throw std::range_error("numeric::binomial(): don't know how to evaluate that.");
 }
 
 
@@ -1810,7 +1810,7 @@ const numeric mod(const numeric &a, const numeric &b)
 /** Modulus (in symmetric representation).
  *  Equivalent to Maple's mods.
  *
- *  @return a mod b in the range [-iquo(abs(m)-1,2), iquo(abs(m),2)]. */
+ *  @return a mod b in the range [-iquo(abs(b)-1,2), iquo(abs(b),2)]. */
 const numeric smod(const numeric &a, const numeric &b)
 {
 	if (a.is_integer() && b.is_integer()) {
