@@ -222,12 +222,14 @@ bool power::info(unsigned inf) const
 		case info_flags::cinteger_polynomial:
 		case info_flags::rational_polynomial:
 		case info_flags::crational_polynomial:
-			return exponent.info(info_flags::nonnegint);
+			return exponent.info(info_flags::nonnegint) &&
+			       basis.info(inf);
 		case info_flags::rational_function:
-			return exponent.info(info_flags::integer);
+			return exponent.info(info_flags::integer) &&
+			       basis.info(inf);
 		case info_flags::algebraic:
-			return (!exponent.info(info_flags::integer) ||
-					basis.info(inf));
+			return !exponent.info(info_flags::integer) ||
+			       basis.info(inf);
 	}
 	return inherited::info(inf);
 }
@@ -246,7 +248,14 @@ ex power::op(size_t i) const
 
 ex power::map(map_function & f) const
 {
-	return (new power(f(basis), f(exponent)))->setflag(status_flags::dynallocated);
+	const ex &mapped_basis = f(basis);
+	const ex &mapped_exponent = f(exponent);
+
+	if (!are_ex_trivially_equal(basis, mapped_basis)
+	 || !are_ex_trivially_equal(exponent, mapped_exponent))
+		return (new power(mapped_basis, mapped_exponent))->setflag(status_flags::dynallocated);
+	else
+		return *this;
 }
 
 int power::degree(const ex & s) const
