@@ -3,7 +3,7 @@
  *  Interface to GiNaC's symbolic exponentiation (basis^exponent). */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,35 +41,36 @@ class power : public basic
 	
 // member functions
 	
-	// other ctors
+	// other constructors
 public:
 	power(const ex & lh, const ex & rh) : inherited(TINFO_power), basis(lh), exponent(rh) {}
 	template<typename T> power(const ex & lh, const T & rh) : inherited(TINFO_power), basis(lh), exponent(rh) {}
 	
 	// functions overriding virtual functions from base classes
 public:
-	void print(const print_context & c, unsigned level = 0) const;
-	unsigned precedence(void) const {return 60;}
+	unsigned precedence() const {return 60;}
 	bool info(unsigned inf) const;
-	unsigned nops() const;
-	ex & let_op(int i);
+	size_t nops() const;
+	ex op(size_t i) const;
 	ex map(map_function & f) const;
 	int degree(const ex & s) const;
 	int ldegree(const ex & s) const;
 	ex coeff(const ex & s, int n = 1) const;
 	ex eval(int level=0) const;
 	ex evalf(int level=0) const;
-	ex evalm(void) const;
+	ex evalm() const;
 	ex series(const relational & s, int order, unsigned options = 0) const;
-	ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;
-	ex normal(lst &sym_lst, lst &repl_lst, int level = 0) const;
-	ex to_rational(lst &repl_lst) const;
-	exvector get_free_indices(void) const;
-	ex simplify_ncmul(const exvector & v) const;
+	ex subs(const exmap & m, unsigned options = 0) const;
+	ex normal(exmap & repl, exmap & rev_lookup, int level = 0) const;
+	ex to_rational(exmap & repl) const;
+	ex to_polynomial(exmap & repl) const;
+	exvector get_free_indices() const;
+	ex conjugate() const;
 protected:
 	ex derivative(const symbol & s) const;
-	unsigned return_type(void) const;
-	unsigned return_type_tinfo(void) const;
+	ex eval_ncmul(const exvector & v) const;
+	unsigned return_type() const;
+	unsigned return_type_tinfo() const;
 	ex expand(unsigned options = 0) const;
 	
 	// new virtual functions which can be overridden by derived classes
@@ -77,9 +78,16 @@ protected:
 	
 	// non-virtual functions in this class
 protected:
-	ex expand_add(const add & a, int n) const;
-	ex expand_add_2(const add & a) const;
-	ex expand_mul(const mul & m, const numeric & n) const;
+	void print_power(const print_context & c, const char *powersymbol, const char *openbrace, const char *closebrace, unsigned level) const;
+	void do_print_dflt(const print_dflt & c, unsigned level) const;
+	void do_print_latex(const print_latex & c, unsigned level) const;
+	void do_print_csrc(const print_csrc & c, unsigned level) const;
+	void do_print_python(const print_python & c, unsigned level) const;
+	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
+
+	ex expand_add(const add & a, int n, unsigned options) const;
+	ex expand_add_2(const add & a, unsigned options) const;
+	ex expand_mul(const mul & m, const numeric & n, unsigned options, bool from_expand = false) const;
 	
 // member variables
 	
