@@ -3,7 +3,7 @@
  *  Interface to class for extended truncated power series. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,17 +36,15 @@ class pseries : public basic
 {
 	GINAC_DECLARE_REGISTERED_CLASS(pseries, basic)
 
-	// other ctors
+	// other constructors
 public:
 	pseries(const ex &rel_, const epvector &ops_);
 
 	// functions overriding virtual functions from base classes
 public:
-	void print(const print_context & c, unsigned level = 0) const;
-	unsigned precedence(void) const {return 38;} // for clarity just below add::precedence
-	unsigned nops(void) const;
-	ex op(int i) const;
-	ex &let_op(int i);
+	unsigned precedence() const {return 38;} // for clarity just below add::precedence
+	size_t nops() const;
+	ex op(size_t i) const;
 	int degree(const ex &s) const;
 	int ldegree(const ex &s) const;
 	ex coeff(const ex &s, int n = 1) const;
@@ -54,19 +52,20 @@ public:
 	ex eval(int level=0) const;
 	ex evalf(int level=0) const;
 	ex series(const relational & r, int order, unsigned options = 0) const;
-	ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;
-	ex normal(lst &sym_lst, lst &repl_lst, int level = 0) const;
+	ex subs(const exmap & m, unsigned options = 0) const;
+	ex normal(exmap & repl, exmap & rev_lookup, int level = 0) const;
 	ex expand(unsigned options = 0) const;
+	ex conjugate() const;
 protected:
 	ex derivative(const symbol & s) const;
 
 	// non-virtual functions in this class
 public:
 	/** Get the expansion variable. */
-	ex get_var(void) const {return var;}
+	ex get_var() const {return var;}
 
 	/** Get the expansion point. */
-	ex get_point(void) const {return point;}
+	ex get_point() const {return point;}
 
 	/** Convert the pseries object to an ordinary polynomial.
 	 *
@@ -78,11 +77,11 @@ public:
 	bool is_compatible_to(const pseries &other) const {return var.is_equal(other.var) && point.is_equal(other.point);}
 
 	/** Check whether series has the value zero. */
-	bool is_zero(void) const {return seq.size() == 0;}
+	bool is_zero() const {return seq.size() == 0;}
 
 	/** Returns true if there is no order term, i.e. the series terminates and
 	 *  false otherwise. */
-	bool is_terminating(void) const;
+	bool is_terminating() const;
 
 	ex add_series(const pseries &other) const;
 	ex mul_const(const numeric &other) const;
@@ -90,6 +89,14 @@ public:
 	ex power_const(const numeric &p, int deg) const;
 	pseries shift_exponents(int deg) const;
 
+protected:
+	void print_series(const print_context & c, const char *openbrace, const char *closebrace, const char *mul_sym, const char *pow_sym, unsigned level) const;
+	void do_print(const print_context & c, unsigned level) const;
+	void do_print_latex(const print_latex & c, unsigned level) const;
+	void do_print_tree(const print_tree & c, unsigned level) const;
+	void do_print_python(const print_python & c, unsigned level) const;
+	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
+
 protected:
 	/** Vector of {coefficient, power} pairs */
 	epvector seq;
