@@ -3,7 +3,7 @@
  *  GiNaC's class registrar (for class basic and all classes derived from it). */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,37 +21,21 @@
  */
 
 #include <string>
+#include <map>
 #include <stdexcept>
 
 #include "registrar.h"
 
 namespace GiNaC {
 
-registered_class_info *first_registered_class = NULL;
-
-/** Find registered_class_info strucure by class name. */
-static inline registered_class_info *find_registered_class_info(const std::string &class_name)
+unsigned find_tinfo_key(const std::string &class_name)
 {
-	registered_class_info *p = first_registered_class;
-	while (p) {
-		if (class_name == p->name)
-			return p;
-		p = p->next;
-	}
-	throw (std::runtime_error("class '" + class_name + "' not registered"));
-}
-
-unsigned int find_tinfo_key(const std::string &class_name)
-{
-	registered_class_info *p = find_registered_class_info(class_name);
-	return p->tinfo_key;
+	return registered_class_info::find(class_name)->options.get_id();
 }
 
 unarch_func find_unarch_func(const std::string &class_name)
 {
-	registered_class_info *p = find_registered_class_info(class_name);
-	return p->unarchive;
+	return registered_class_info::find(class_name)->options.get_unarch_func();
 }
 
-
 } // namespace GiNaC
