@@ -38,6 +38,7 @@ const unsigned TINFO_exprseq       = 0x00030001U;
 const unsigned TINFO_function      = 0x00031001U;
 const unsigned TINFO_fderivative   = 0x00032001U;
 const unsigned TINFO_ncmul         = 0x00031002U;
+const unsigned TINFO_integral      = 0x00033001U;
 
 const unsigned TINFO_lst           = 0x00040001U;
 
@@ -81,6 +82,7 @@ const unsigned TINFO_su3f          = 0x000e100aU;
 const unsigned TINFO_su3d          = 0x000e100bU;
 const unsigned TINFO_diracone      = 0x000e100cU;
 const unsigned TINFO_diracgamma    = 0x000e100dU;
+const unsigned TINFO_cliffordunit  = 0x000e101dU;
 const unsigned TINFO_diracgamma5   = 0x000e100eU;
 const unsigned TINFO_diracgammaL   = 0x000e100fU;
 const unsigned TINFO_diracgammaR   = 0x000e1010U;
