@@ -4,7 +4,7 @@
  *  internal header file. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ template<class T>
 std::string ToString(const T & t)
 {
 	std::ostringstream buf;
-	buf << t << std::ends;
+	buf << t;
 	return buf.str();
 }
 
