@@ -119,4 +119,14 @@ bool wildcard::match(const ex & pattern, lst & repl_lst) const
 	return is_equal(ex_to<basic>(pattern));
 }
 
+bool haswild(const ex & x)
+{
+	if (is_a<wildcard>(x))
+		return true;
+	for (int i=0; i<x.nops(); ++i)
+		if (haswild(x.op(i)))
+			return true;
+	return false;
+}
+
 } // namespace GiNaC
