@@ -2,7 +2,7 @@
  *
  *  Global definitions for ginsh.
  *
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -73,4 +73,7 @@ extern char **file_list;
 typedef map<string, symbol> sym_tab;
 extern sym_tab syms;
 
+// Type of symbols to generate (real or complex)
+extern unsigned symboltype;
+
 #endif
