@@ -5,7 +5,7 @@
  *  functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2002 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 
 // Table of names and descriptors of functions to be added
 static const fcn_init extended_fcns[] = {
-	{NULL, fcn_desc(f_dummy, 0)} // End marker
+	{NULL, f_dummy, 0} // End marker
 };
 
 // Table of help strings for functions
