@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -110,7 +110,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 29 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 29 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
 
 #include "config.h"
 
@@ -135,6 +135,12 @@ static char *orig_basic_word_break_characters;
 static const char *orig_basic_word_break_characters;
 #endif
 
+#if (GINAC_RL_VERSION_MAJOR >= 5)
+#define GINAC_RL_COMPLETER_CAST(a) const_cast<char *>((a))
+#else
+#define GINAC_RL_COMPLETER_CAST(a) (a)
+#endif
+
 // Expression stack for %, %% and %%%
 static void push(const ex &e);
 static ex exstack[3];
@@ -198,22 +204,29 @@ typedef int YYSTYPE;
 
 
 /* Line 214 of yacc.c.  */
-#line 201 "ginsh_parser.cc"
+#line 208 "ginsh_parser.cc"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# if YYSTACK_USE_ALLOCA
-#  define YYSTACK_ALLOC alloca
+# ifdef YYSTACK_USE_ALLOCA
+#  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
 # else
-#  ifndef YYSTACK_USE_ALLOCA
-#   if defined (alloca) || defined (_ALLOCA_H)
-#    define YYSTACK_ALLOC alloca
-#   else
-#    ifdef __GNUC__
-#     define YYSTACK_ALLOC __builtin_alloca
-#    endif
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
+#   ifdef __GNUC__
+#    define YYSTACK_ALLOC __builtin_alloca
 #   endif
 #  endif
 # endif
@@ -226,15 +239,15 @@ typedef int YYSTYPE;
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC malloc
-#  define YYSTACK_FREE free
+#  define YYSTACK_ALLOC YYMALLOC
+#  define YYSTACK_FREE YYFREE
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYSTYPE_IS_TRIVIAL)))
+	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
@@ -255,7 +268,7 @@ union yyalloc
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if 1 < __GNUC__
+#  if defined (__GNUC__) && 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -393,13 +406,13 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   116,   116,   117,   120,   121,   130,   138,   146,   160,
-     168,   176,   177,   178,   179,   180,   181,   182,   183,   184,
-     197,   198,   199,   200,   205,   206,   207,   207,   214,   215,
-     218,   219,   220,   221,   222,   223,   224,   225,   226,   234,
-     235,   236,   237,   238,   239,   240,   241,   242,   243,   244,
-     245,   246,   247,   248,   249,   250,   251,   252,   255,   256,
-     259,   260,   263,   264,   267,   268,   271,   272
+       0,   122,   122,   123,   126,   127,   136,   144,   152,   166,
+     174,   182,   183,   184,   185,   186,   187,   188,   189,   190,
+     203,   204,   205,   206,   211,   212,   213,   213,   220,   221,
+     224,   225,   226,   227,   228,   229,   230,   231,   232,   240,
+     241,   242,   243,   244,   245,   246,   247,   248,   249,   250,
+     251,   252,   253,   254,   255,   256,   257,   258,   261,   262,
+     265,   266,   269,   270,   273,   274,   277,   278
 };
 #endif
 
@@ -408,15 +421,15 @@ static const unsigned short yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "T_NUMBER", "T_SYMBOL", "T_LITERAL", 
-  "T_DIGITS", "T_QUOTE", "T_QUOTE2", "T_QUOTE3", "T_EQUAL", "T_NOTEQ", 
-  "T_LESSEQ", "T_GREATEREQ", "T_QUIT", "T_WARRANTY", "T_PRINT", 
-  "T_IPRINT", "T_PRINTLATEX", "T_PRINTCSRC", "T_TIME", "T_XYZZY", 
-  "T_INVENTORY", "T_LOOK", "T_SCORE", "T_COMPLEX_SYMBOLS", 
-  "T_REAL_SYMBOLS", "'='", "'<'", "'>'", "'+'", "'-'", "'*'", "'/'", 
-  "NEG", "'^'", "'!'", "';'", "':'", "'('", "')'", "'?'", "'''", "'{'", 
-  "'}'", "'['", "']'", "','", "$accept", "input", "line", "@1", "exp", 
-  "exprseq", "list_or_empty", "list", "matrix", "row", 0
+  "$end", "error", "$undefined", "T_NUMBER", "T_SYMBOL", "T_LITERAL",
+  "T_DIGITS", "T_QUOTE", "T_QUOTE2", "T_QUOTE3", "T_EQUAL", "T_NOTEQ",
+  "T_LESSEQ", "T_GREATEREQ", "T_QUIT", "T_WARRANTY", "T_PRINT", "T_IPRINT",
+  "T_PRINTLATEX", "T_PRINTCSRC", "T_TIME", "T_XYZZY", "T_INVENTORY",
+  "T_LOOK", "T_SCORE", "T_COMPLEX_SYMBOLS", "T_REAL_SYMBOLS", "'='", "'<'",
+  "'>'", "'+'", "'-'", "'*'", "'/'", "NEG", "'^'", "'!'", "';'", "':'",
+  "'('", "')'", "'?'", "'''", "'{'", "'}'", "'['", "']'", "','", "$accept",
+  "input", "line", "@1", "exp", "exprseq", "list_or_empty", "list",
+  "matrix", "row", 0
 };
 #endif
 
@@ -626,7 +639,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrlab1
+#define YYERROR		goto yyerrorlab
 
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
@@ -661,11 +674,11 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)         \
-  Current.first_line   = Rhs[1].first_line;      \
-  Current.first_column = Rhs[1].first_column;    \
-  Current.last_line    = Rhs[N].last_line;       \
-  Current.last_column  = Rhs[N].last_column;
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
@@ -709,7 +722,7 @@ do {								\
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (cinluded).                                                   |
+| TOP (included).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
@@ -749,9 +762,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylineno = yyrline[yyrule];
+  unsigned int yylno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylineno);
+             yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -788,7 +801,7 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
-#if YYMAXDEPTH == 0
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
 # undef YYMAXDEPTH
 #endif
 
@@ -1211,7 +1224,7 @@ yyreduce:
   switch (yyn)
     {
         case 5:
-#line 121 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 127 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			cout << yyvsp[-1] << endl;
@@ -1224,7 +1237,7 @@ yyreduce:
     break;
 
   case 6:
-#line 130 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 136 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			push(yyvsp[-1]);
@@ -1236,7 +1249,7 @@ yyreduce:
     break;
 
   case 7:
-#line 138 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 144 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			yyvsp[-2].print(print_tree(std::cout));
@@ -1248,7 +1261,7 @@ yyreduce:
     break;
 
   case 8:
-#line 146 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 152 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			ex e = yyvsp[-2];
@@ -1266,7 +1279,7 @@ yyreduce:
     break;
 
   case 9:
-#line 160 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 166 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			yyvsp[-2].print(print_latex(std::cout)); cout << endl;
@@ -1278,7 +1291,7 @@ yyreduce:
     break;
 
   case 10:
-#line 168 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 174 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		try {
 			yyvsp[-2].print(print_csrc_double(std::cout)); cout << endl;
@@ -1290,47 +1303,47 @@ yyreduce:
     break;
 
   case 11:
-#line 176 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 182 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help(ex_to<symbol>(yyvsp[0]).get_name());}
     break;
 
   case 12:
-#line 177 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 183 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help("time");}
     break;
 
   case 13:
-#line 178 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 184 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help("print");}
     break;
 
   case 14:
-#line 179 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 185 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help("iprint");}
     break;
 
   case 15:
-#line 180 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 186 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help("print_latex");}
     break;
 
   case 16:
-#line 181 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 187 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help("print_csrc");}
     break;
 
   case 17:
-#line 182 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 188 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {print_help_topics();}
     break;
 
   case 18:
-#line 183 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 189 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {YYACCEPT;}
     break;
 
   case 19:
-#line 184 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 190 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		cout << "This program is free software; you can redistribute it and/or modify it under\n";
 		cout << "the terms of the GNU General Public License as published by the Free Software\n";
@@ -1347,22 +1360,22 @@ yyreduce:
     break;
 
   case 20:
-#line 197 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 203 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {cout << "Nothing happens.\n";}
     break;
 
   case 21:
-#line 198 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 204 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {cout << "You're not carrying anything.\n";}
     break;
 
   case 22:
-#line 199 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 205 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {cout << "You're in a twisty little maze of passages, all alike.\n";}
     break;
 
   case 23:
-#line 200 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 206 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		cout << "If you were to quit now, you would score ";
 		cout << (syms.size() > 350 ? 350 : syms.size());
@@ -1371,22 +1384,22 @@ yyreduce:
     break;
 
   case 24:
-#line 205 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 211 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     { symboltype = domain::real; }
     break;
 
   case 25:
-#line 206 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 212 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     { symboltype = domain::complex; }
     break;
 
   case 26:
-#line 207 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 213 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {getrusage(RUSAGE_SELF, &start_time);}
     break;
 
   case 27:
-#line 207 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 213 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		getrusage(RUSAGE_SELF, &end_time);
 		cout << (end_time.ru_utime.tv_sec - start_time.ru_utime.tv_sec) +
@@ -1397,57 +1410,57 @@ yyreduce:
     break;
 
   case 28:
-#line 214 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 220 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyclearin; yyerrok;}
     break;
 
   case 29:
-#line 215 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 221 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyclearin; yyerrok;}
     break;
 
   case 30:
-#line 218 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 224 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 31:
-#line 219 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 225 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0].eval();}
     break;
 
   case 32:
-#line 220 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 226 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-1];}
     break;
 
   case 33:
-#line 221 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 227 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 34:
-#line 222 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 228 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 35:
-#line 223 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 229 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = exstack[0];}
     break;
 
   case 36:
-#line 224 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 230 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = exstack[1];}
     break;
 
   case 37:
-#line 225 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 231 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = exstack[2];}
     break;
 
   case 38:
-#line 226 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 232 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {
 		fcn_tab::const_iterator i = find_function(yyvsp[-3], yyvsp[-1].nops());
 		if (i->second.is_ginac) {
@@ -1459,155 +1472,155 @@ yyreduce:
     break;
 
   case 39:
-#line 234 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 240 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0]; Digits = ex_to<numeric>(yyvsp[0]).to_int();}
     break;
 
   case 40:
-#line 235 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 241 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0]; const_cast<symbol&>(ex_to<symbol>(yyvsp[-2])).assign(yyvsp[0]);}
     break;
 
   case 41:
-#line 236 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 242 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] == yyvsp[0];}
     break;
 
   case 42:
-#line 237 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 243 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] != yyvsp[0];}
     break;
 
   case 43:
-#line 238 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 244 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] < yyvsp[0];}
     break;
 
   case 44:
-#line 239 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 245 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] <= yyvsp[0];}
     break;
 
   case 45:
-#line 240 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 246 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] > yyvsp[0];}
     break;
 
   case 46:
-#line 241 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 247 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] >= yyvsp[0];}
     break;
 
   case 47:
-#line 242 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 248 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] + yyvsp[0];}
     break;
 
   case 48:
-#line 243 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 249 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] - yyvsp[0];}
     break;
 
   case 49:
-#line 244 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 250 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] * yyvsp[0];}
     break;
 
   case 50:
-#line 245 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 251 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-2] / yyvsp[0];}
     break;
 
   case 51:
-#line 246 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 252 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = -yyvsp[0];}
     break;
 
   case 52:
-#line 247 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 253 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 53:
-#line 248 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 254 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = power(yyvsp[-2], yyvsp[0]);}
     break;
 
   case 54:
-#line 249 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 255 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = factorial(yyvsp[-1]);}
     break;
 
   case 55:
-#line 250 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 256 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-1];}
     break;
 
   case 56:
-#line 251 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 257 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[-1];}
     break;
 
   case 57:
-#line 252 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 258 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = lst_to_matrix(ex_to<lst>(yyvsp[-1]));}
     break;
 
   case 58:
-#line 255 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 261 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = exprseq(yyvsp[0]);}
     break;
 
   case 59:
-#line 256 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 262 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {exprseq es(ex_to<exprseq>(yyvsp[-2])); yyval = es.append(yyvsp[0]);}
     break;
 
   case 60:
-#line 259 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 265 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = *new lst;}
     break;
 
   case 61:
-#line 260 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 266 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = yyvsp[0];}
     break;
 
   case 62:
-#line 263 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 269 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = lst(yyvsp[0]);}
     break;
 
   case 63:
-#line 264 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 270 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 
   case 64:
-#line 267 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 273 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = lst(yyvsp[-1]);}
     break;
 
   case 65:
-#line 268 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 274 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-4])); yyval = l.append(yyvsp[-1]);}
     break;
 
   case 66:
-#line 271 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 277 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {yyval = lst(yyvsp[0]);}
     break;
 
   case 67:
-#line 272 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 278 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
     {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 
 
     }
 
-/* Line 999 of yacc.c.  */
-#line 1610 "ginsh_parser.cc"
+/* Line 1000 of yacc.c.  */
+#line 1624 "ginsh_parser.cc"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1648,18 +1661,33 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
+	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx, yycount;
+	  int yyx;
 
-	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  for (yyx = yyn < 0 ? -yyn : 0;
-	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
+	  int yyxbegin = yyn < 0 ? -yyn : 0;
+
+	  /* Stay within bounds of both yycheck and yytname.  */
+	  int yychecklim = YYLAST - yyn;
+	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+	  int yycount = 0;
+
+	  yyprefix = ", expecting ";
+	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("syntax error, unexpected ") + 1;
-	  yysize += yystrlen (yytname[yytype]);
+	      {
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
+		  {
+		    yysize = 0;
+		    break;
+		  }
+	      }
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1668,16 +1696,13 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yycount = 0;
-		  for (yyx = yyn < 0 ? -yyn : 0;
-		       yyx < (int) (sizeof (yytname) / sizeof (char *));
-		       yyx++)
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			const char *yyq = ! yycount ? ", expecting " : " or ";
-			yyp = yystpcpy (yyp, yyq);
+			yyp = yystpcpy (yyp, yyprefix);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yycount++;
+			yyprefix = " or ";
 		      }
 		}
 	      yyerror (yymsg);
@@ -1698,25 +1723,27 @@ yyerrlab:
       /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      /* Return failure if at end of input.  */
-      if (yychar == YYEOF)
+      if (yychar <= YYEOF)
         {
-	  /* Pop the error token.  */
-          YYPOPSTACK;
-	  /* Pop the rest of the stack.  */
-	  while (yyss < yyssp)
-	    {
-	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
-	      yydestruct (yystos[*yyssp], yyvsp);
-	      YYPOPSTACK;
-	    }
-	  YYABORT;
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
+	  if (yychar == YYEOF)
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
+      else
+	{
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
+	  yychar = YYEMPTY;
 
-      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
-      yydestruct (yytoken, &yylval);
-      yychar = YYEMPTY;
-
+	}
     }
 
   /* Else will try to reuse lookahead token after shifting the error
@@ -1724,9 +1751,27 @@ yyerrlab:
   goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
+/*---------------------------------------------------.
+| yyerrorlab -- error raised explicitly by YYERROR.  |
+`---------------------------------------------------*/
+yyerrorlab:
+
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
+  if (0)
+     goto yyerrorlab;
+#endif
+
+  yyvsp -= yylen;
+  yyssp -= yylen;
+  yystate = *yyssp;
+  goto yyerrlab1;
+
+
+/*-------------------------------------------------------------.
+| yyerrlab1 -- common code for both syntax error and YYERROR.  |
+`-------------------------------------------------------------*/
 yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
@@ -1750,9 +1795,8 @@ yyerrlab1:
 
       YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
       yydestruct (yystos[yystate], yyvsp);
-      yyvsp--;
-      yystate = *--yyssp;
-
+      YYPOPSTACK;
+      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
@@ -1801,7 +1845,7 @@ yyreturn:
 }
 
 
-#line 280 "/home/cbauer/projects/GiNaC-1.2/ginsh/ginsh_parser.yy"
+#line 286 "/home/cbauer/projects/GiNaC-HEAD/ginsh/ginsh_parser.yy"
 
 // Error print routine
 int yyerror(char *s)
@@ -1929,6 +1973,11 @@ static ex f_find(const exprseq &e)
 	return found;
 }
 
+static ex f_integer_content(const exprseq &e)
+{
+	return e[0].expand().integer_content();
+}
+
 static ex f_inverse(const exprseq &e)
 {
 	CHECK_ARG(0, matrix, inverse);
@@ -1994,11 +2043,23 @@ static ex f_quo(const exprseq &e)
 	return quo(e[0], e[1], e[2]);
 }
 
+static ex f_rank(const exprseq &e)
+{
+	CHECK_ARG(0, matrix, rank);
+	return ex_to<matrix>(e[0]).rank();
+}
+
 static ex f_rem(const exprseq &e)
 {
 	return rem(e[0], e[1], e[2]);
 }
 
+static ex f_resultant(const exprseq &e)
+{
+	CHECK_ARG(2, symbol, resultant);
+	return resultant(e[0], e[1], ex_to<symbol>(e[2]));
+}
+
 static ex f_series(const exprseq &e)
 {
 	CHECK_ARG(2, numeric, series);
@@ -2084,6 +2145,7 @@ static const fcn_init builtin_fcns[] = {
 	{"find", f_find, 2},
 	{"gcd", f_gcd, 2},
 	{"has", f_has, 2},
+	{"integer_content", f_integer_content, 1},
 	{"inverse", f_inverse, 1},
 	{"iprint", f_dummy, 0},      // for Tab-completion
 	{"is", f_is, 1},
@@ -2106,7 +2168,9 @@ static const fcn_init builtin_fcns[] = {
 	{"print_csrc", f_dummy, 0},  // for Tab-completion
 	{"print_latex", f_dummy, 0}, // for Tab-completion
 	{"quo", f_quo, 3},
+	{"rank", f_rank, 1},
 	{"rem", f_rem, 3},
+	{"resultant", f_resultant, 3},
 	{"series", f_series, 3},
 	{"sprem", f_sprem, 3},
 	{"sqrfree", f_sqrfree1, 1},
@@ -2322,7 +2386,7 @@ static char **fcn_completion(const char *text, int start, int end)
 		// For shell commands, revert back to filename completion
 		rl_completion_append_character = orig_completion_append_character;
 		rl_basic_word_break_characters = orig_basic_word_break_characters;
-		rl_completer_word_break_characters = rl_basic_word_break_characters;
+		rl_completer_word_break_characters = GINAC_RL_COMPLETER_CAST(rl_basic_word_break_characters);
 #if (GINAC_RL_VERSION_MAJOR < 4) || (GINAC_RL_VERSION_MAJOR == 4 && GINAC_RL_VERSION_MINOR < 2)
 		return completion_matches(const_cast<char *>(text), (CPFunction *)filename_completion_function);
 #else
@@ -2332,7 +2396,7 @@ static char **fcn_completion(const char *text, int start, int end)
 		// Otherwise, complete function names
 		rl_completion_append_character = '(';
 		rl_basic_word_break_characters = " \t\n\"#$%&'()*+,-./:;<=>?@[\\]^`{|}~";
-		rl_completer_word_break_characters = rl_basic_word_break_characters;
+		rl_completer_word_break_characters = GINAC_RL_COMPLETER_CAST(rl_basic_word_break_characters);
 #if (GINAC_RL_VERSION_MAJOR < 4) || (GINAC_RL_VERSION_MAJOR == 4 && GINAC_RL_VERSION_MINOR < 2)
 		return completion_matches(const_cast<char *>(text), (CPFunction *)fcn_generator);
 #else
